# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
The :xfile:`models` module for :mod:`lino_book.projects.belref`.

"""

from lino.api import dd

concepts = dd.resolve_app('concepts')


class Main(concepts.TopLevelConcepts):
    pass


@dd.receiver(dd.post_analyze)
def my_details(sender, **kw):
    site = sender

    lst = (site.models.countries.Places,
           site.models.countries.Countries,
           site.models.concepts.Concepts)
    for t in lst:
        t.required_roles.discard(dd.SiteUser)
        t.required_roles.discard(dd.SiteStaff)

    site.models.countries.Places.set_detail_layout("""
    name country inscode
    parent type id
    PlacesByPlace
    """)

    site.models.countries.Countries.set_detail_layout("""
    isocode name short_code inscode
    countries.PlacesByCountry
    """)



# -*- coding: UTF-8 -*-
"""
The base settings module for :mod:`lino_book.projects.roger`.
"""

from __future__ import unicode_literals
from __future__ import print_function

from lino_voga.lib.voga.settings import *


class Site(Site):

    # default_ui = 'lino_extjs6.extjs6'

    title = "Lino Voga for Roger"
    languages = "en de fr"

    demo_fixtures = """std minimal_ledger 
    demo voga demo_bookings payments demo2 checkdata""".split()

    def get_plugin_configs(self):
        yield super(Site, self).get_plugin_configs()
        yield ('countries', 'hide_region', True)
        yield ('countries', 'country_code', 'BE')
        yield ('vat', 'declaration_plugin', 'lino_xl.lib.bevats')
        yield ('ledger', 'use_pcmn', True)
        yield ('ledger', 'start_year', 2014)

    def get_apps_modifiers(self, **kw):
        kw = super(Site, self).get_apps_modifiers(**kw)
        # alternative implementations:
        kw.update(courses='lino_voga.lib.roger.courses')
        return kw



# -*- coding: UTF-8 -*-
# Copyright 2012 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

r"""
Quantity Fields
===============

See also :mod:`lino.utils.quantity`.

>>> from lino import startup
>>> startup('lino_book.projects.quantityfield.settings')

Create some data:

>>> Item(name="Screw",price='2.49',qty='5').save()
>>> Item(name="Leather",price='24.95',qty='2.39').save()
>>> Item(name="Work",price='59.99',qty='0:40').save()

Here is our data:

>>> print '\n'.join([str(i) for i in Item.objects.all()])
5 x Screw at 2.49 = 12.45 EUR
2.39 x Leather at 24.95 = 59.6305 EUR
0:40 x Work at 59.99 = 39:60 EUR

Default value for `qty` is 1, while `discount` is "pseudo-nullable".

>>> Item(name="Thing",price='59.99').save()
>>> print str(Item.objects.get(pk=4))
1 x Thing at 59.99 = 59.99 EUR

>>> Item(name="Another thing",price='59.99',discount='10%').save()
>>> print str(Item.objects.get(pk=5))
1 x Another thing at 59.99 (-10%) = 53.991 EUR


"""

from __future__ import unicode_literals


from django.db import models
from lino.api import dd



class Item(dd.Model):
    name = models.CharField(max_length=50)
    qty = dd.QuantityField(default='1')
    discount = dd.QuantityField()
    price = dd.PriceField()

    def __str__(self):
        if self.discount is None:
            return "%s x %s at %s = %s EUR" % (self.qty, self.name, self.price, self.total())
        else:
            return "%s x %s at %s (-%s) = %s EUR" % (self.qty, self.name, self.price, self.discount, self.total())

    def total(self):
        if self.discount is None:
            return self.qty * self.price
        else:
            return self.qty * (1 - self.discount) * self.price



from django.db import models
from django.contrib.contenttypes.models import ContentType
from lino.api import dd
from lino.core.gfks import GenericForeignKey



class Member(dd.Model):
    name = models.CharField(max_length=200)
    email = models.EmailField(max_length=200, blank=True)

    def __str__(self):
        return self.name



class Comment(dd.Model):
    allow_cascaded_delete = ['owner']
    owner_type = dd.ForeignKey(ContentType)
    owner_id = models.PositiveIntegerField()
    owner = GenericForeignKey('owner_type', 'owner_id')
    text = models.CharField(max_length=200)

    def __str__(self):
        return '%s object' % (self.__class__.__name__)



class Note(dd.Model):
    owner_type = dd.ForeignKey(ContentType)
    owner_id = models.PositiveIntegerField()
    owner = GenericForeignKey('owner_type', 'owner_id')
    text = models.CharField(max_length=200)

    def __str__(self):
        return '%s object' % (self.__class__.__name__)



class Memo(dd.Model):
    owner_type = dd.ForeignKey(ContentType, blank=True, null=True)
    owner_id = models.PositiveIntegerField(blank=True, null=True)
    owner = GenericForeignKey('owner_type', 'owner_id')
    text = models.CharField(max_length=200)

    def __str__(self):
        return '%s object' % (self.__class__.__name__)



from lino.api.shell import *
from lino.utils.instantiator import Instantiator


def objects():
    author = Instantiator(
        'tables.Author', 'first_name last_name country').build
    adams = author("Douglas", "Adams", "UK")
    yield adams
    camus = author("Albert", "Camus", "FR")
    yield camus
    huttner = author("Hannes", "Huttner", "DE")
    yield huttner

    book = Instantiator('tables.Book', 'title author published price').build
    yield book("Last chance to see...", adams, 1990, '9.90')
    yield book("The Hitchhiker's Guide to the Galaxy", adams, 1978, '19.90')
    yield book("Das Blaue vom Himmel", huttner, 1975, '14.90')
    yield book("L'etranger", camus, 1957, '6.90')
    # yield book("Book", camus, 2001, '4.90')




# -*- coding: utf-8 -*-
# Copyright 2014-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""This module contains some quick tests:

- In a :class:`lino.modlib.sepa.models.Account`:

  - Fill IBAN and BIC from Belgian NBAN or IBAN
  - Test whether the record is being validated.


You can run only these tests by issuing::

  $ cd lino/tutorials/mini
  $ python manage.py test

"""

from __future__ import unicode_literals
from __future__ import print_function

import logging
logger = logging.getLogger(__name__)

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase


class QuickTest(RemoteAuthTestCase):
    maxDiff = None

    def test_sepa(self):
        from lino.api.shell import sepa, contacts
        partner = contacts.Partner(name="Foo")
        partner.full_clean()
        partner.save()

        # Fill IBAN and BIC from Belgian NBAN
        obj = sepa.Account(partner=partner, iban="001-1148294-84")
        obj.full_clean()
        self.assertEqual(obj.bic, 'GEBABEBB')
        self.assertEqual(obj.iban, 'BE03001114829484')

        # Fill BIC from Belgian IBAN
        obj = sepa.Account(partner=partner, iban="BE03001114829484")
        obj.full_clean()
        self.assertEqual(obj.bic, 'GEBABEBB')
        self.assertEqual(obj.iban, 'BE03001114829484')

        # Raise ValidationError when invalid IBAN
        obj = sepa.Account(partner=partner, iban="BE03001114829483")
        try:
            obj.full_clean()
            self.fail("Expected ValidationError")
        except ValidationError as e:
            self.assertEqual(
                e.message_dict,
                {'iban': ['Not a valid IBAN.']})

        # Raise ValidationError when invalid BIC
        obj = sepa.Account(
            partner=partner, iban="BE03001114829484", bic="FOO")
        try:
            obj.full_clean()
            self.fail("Expected ValidationError")
        except ValidationError as e:
            self.assertEqual(
                e.message_dict,
                {'bic': ['BIC codes have either 8 or 11 characters.']})

        # Raise ValidationError when no BIC is given
        obj = sepa.Account(partner=partner)
        try:
            obj.full_clean()
            self.fail("Expected ValidationError")
        except ValidationError as e:
            self.assertEqual(
                e.message_dict,
                {'iban': [u'This field cannot be blank.']})



# don't forget to import the default workflows:
from lino_noi.lib.noi.workflows import *

from django.conf import settings
from lino.modlib.about.choicelists import TimeZones
TimeZones.clear()
add = TimeZones.add_item
add('01', settings.TIME_ZONE or 'UTC', 'default')
add('02', "Europe/Tallinn")
add('03', "Europe/Brussels")
add('04', "Africa/Tunis")



from lino.api import dd


class Questions(dd.Table):
    model = 'polls.Question'
    order_by = ['pub_date']

    detail_layout = """
    id question_text
    hidden pub_date
    ChoicesByQuestion
    """

    insert_layout = """
    question_text
    hidden
    """


class Choices(dd.Table):
    model = 'polls.Choice'


class ChoicesByQuestion(Choices):
    master_key = 'question'




from lino.api import rt


def objects():
    Country = rt.models.combo.Country
    City = rt.models.combo.City
    
    be = Country(name="Belgium")
    yield be
    ee = Country(name="Estonia")
    yield ee
    
    yield City(name="Eupen", country=be)
    yield City(name="Brussels", country=be)
    yield City(name="Gent", country=be)
    yield City(name="Raeren", country=be)
    yield City(name="Namur", country=be)
    
    yield City(name="Tallinn", country=ee)
    yield City(name="Tartu", country=ee)
    yield City(name="Vigala", country=ee)



# -*- coding: utf-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about ticket and vote workflow.

You can run only these tests by issuing::

  $ go team
  $ python manage.py test tests.test_workflow

"""

from __future__ import unicode_literals
from __future__ import print_function

from django.conf import settings
from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import dd, rt

from lino.utils.instantiator import create_row as create
    

class WorkflowTests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        Ticket = rt.models.tickets.Ticket
        # Project = rt.models.tickets.Project
        # Line = rt.models.courses.Line
        # Activity = rt.models.courses.Course
        # Enrolment = rt.models.courses.Enrolment
        Meeting = rt.models.meetings.Meeting
        Change = rt.models.changes.Change
        User = rt.models.users.User
        Vote = rt.models.votes.Vote
        VoteStates = rt.models.votes.VoteStates
        VotesByVotable = rt.models.votes.VotesByVotable
        # ContentType = rt.models.contenttypes.ContentType
        # ct_Ticket = ContentType.objects.get_for_model(Ticket)

        # create(Project, name='project')

        robin = create(User, username='robin',
                       first_name="Robin",
                       user_type=UserTypes.admin,
                       language="en")
        anna = create(User, username='anna',
                      first_name="Anna",
                      user_type=UserTypes.user,
                      language="en")
        berta = create(User, username='berta',
                       first_name="Berta",
                       user_type=UserTypes.user,
                       language="en")
        meeting = create(Meeting, name="Test")
        # sprints = create(Line, name="Sprints")
        # sprint = create(Activity, line=sprints)
        #
        # Enrolment(course=sprint, pupil=robin)
        
        ses = rt.login('robin')
        
        ticket = create(Ticket, summary="First", user=robin)
        ticket.after_ui_save(ses, None)
        vote = Vote.objects.get(votable=ticket)
        self.assertEqual(vote.user, robin)
        self.assertEqual(vote.state, VoteStates.author)
        
        def check_success(ia, **kwargs):
            rv = ia.run_from_session(ses, **kwargs)
            self.assertEqual(rv, {'success': True, 'refresh': True})

        check_success(ticket.mark_talk)
        check_success(ticket.mark_started)
        check_success(ticket.mark_refused)
        check_success(ticket.mark_closed)
        check_success(ticket.mark_refused)
        
        Vote.objects.all().delete()
        Ticket.objects.all().delete()
        
        self.assertEqual(Vote.objects.count(), 0)

        ticket = create(
            Ticket, summary="Second", user=robin, end_user=anna)
        ticket.after_ui_save(ses, None)
        self.assertEqual(Vote.objects.count(), 2)
        
        vote = Vote.objects.get(votable=ticket, user=anna)
        self.assertEqual(vote.state, VoteStates.author)

        
        # manually creating a vote will make the vote invited:
        vote = create(
            Vote, votable=ticket, user=berta)
        # vote.after_ui_save(ses, None)
        self.assertEqual(vote.state, VoteStates.invited)

        # raise Exception("{!r}".format(
        #     VotesByVotable.submit_insert.defining_actor))
        # rv = VotesByVotable.submit_insert.run_from_session(
        #     ses, user=berta)


        # try:
        #     robin.delete()
        #     self.fail("Expected veto")
        # except Warning as e:
        #     self.assertEqual(
        #         str(e), "Cannot delete User Robin "
        #         "because 1 Changes refer to it.")

        



# -*- coding: UTF-8 -*-
# Copyright 2017-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
The settings package.

.. autosummary::
   :toctree:

   demo
   doctests
   memory



"""

from __future__ import print_function
from __future__ import unicode_literals

from lino_tera.lib.tera.settings import *


class Site(Site):

    verbose_name = "Lino Tera for Lydia"
   
    demo_fixtures = 'std minimal_ledger demo demo_bookings payments demo2'.split()

    languages = 'en de fr'
    
# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



# -*- coding: UTF-8 -*-
# Copyright 2012-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""The Django settings module for Lino CMS.

"""

from lino.projects.std.settings import *


class Site(Site):

    verbose_name = "Lino CMS"
    version = "0.1"
    author = 'Rumma & Ko OÜ'
    author_email = 'luc@lino-framework.org'

    default_ui = 'lino_react.react'
    # default_ui = 'lino_xl.lib.pages'
    # admin_ui = 'lino.modlib.extjs'

    languages = 'en de fr'

    # project_model = 'tickets.Project'

    demo_fixtures = ['std', 'demo', 'demo2', 'intro']

    sidebar_width = 3

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.gfks'
        yield 'lino.modlib.extjs'
        yield 'lino.modlib.memo'
        # yield 'lino.modlib.bootstrap3'
        yield 'lino.modlib.users'
        yield 'lino.modlib.publisher'
        yield 'lino_xl.lib.countries'
        yield 'lino_xl.lib.contacts'
        #~ yield 'lino_xl.lib.outbox'
        yield 'lino_xl.lib.blogs'
        # yield 'lino.modlib.tickets'
        yield 'lino_xl.lib.pages'
        # yield 'lino_book.projects.cms'



# -*- coding: UTF-8 -*-
# Copyright 2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals

from lino.projects.std.settings import *

from django.utils.translation import ugettext_lazy as _


class Site(Site):

    verbose_name = "Lino EstRef"
    description = _("Estonian Reference")
    version = "0.1"
    url = "http://www.lino-framework.org/estref.html"
    author = 'Luc Saffre'
    author_email = 'luc.saffre@gmail.com'

    demo_fixtures = 'all_countries est demo2'

    languages = 'et en'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.system'
        yield 'lino_xl.lib.countries'
        # yield 'lino_xl.lib.contacts'
        yield 'lino_book.projects.estref'




# -*- coding: UTF-8 -*-

from __future__ import unicode_literals

from lino.api import dd

Product = dd.resolve_model('mldbc.Product')


def P(en, de, fr, cat, price):
    return Product(
        price=price,
        category=cat,
        **dd.babel_values('name', en=en, de=de, fr=fr))


def objects():
    yield P("Chair", "Stuhl", "Chaise", '03', '29.95')
    yield P("Table", "Tisch", "Table", '03', '89.95')
    # doctests fail with non-ascii text, so we need to cheat:
    # yield P("Monitor", "Bildschirm", "Écran", '01', '19.95')
    yield P("Monitor", "Bildschirm", "Ecran", '01', '19.95')
    yield P("Mouse", "Maus", "Souris", '03', '2.95')
    yield P("Keyboard", "Tastatur", "Clavier", '03', '4.95')
    yield P("Consultation", "Beratung", "Consultation", '02', '59.95')



from lino_book.projects.chatter.settings import *
from lino.utils import i2d


class Site(Site):
    title = Site.verbose_name + " demo"
    the_demo_date = i2d(20141023)

SITE = Site(globals())

DEBUG = True
# the following line should always be commented out in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""This is Luc's personal collection of Belgian vocabulary.  It is
not complete enough to be of real benefit, but publicly available at
http://belref.lino-framework.org/

"""

from __future__ import unicode_literals
import re


abbrRE = re.compile("^(.*)\s*\((.*)\)\s*", re.DOTALL)


from lino.api import dd

Concept = dd.resolve_model('concepts.Concept')
Link = dd.resolve_model('concepts.Link')

from lino_xl.lib.concepts.models import LinkTypes


def C(en, de, fr='', nl='', jargon=None, obsoletes=None, **kw):
    texts = dict(en=en, de=de, fr=fr, nl=nl)
    name = dict()
    abbr = dict()
    for lang in 'en', 'de', 'fr', 'nl':
        t = texts.get(lang)
        if t:
            mo = abbrRE.match(t)
            if mo:
                abbr[lang] = mo.group(1).strip()
                name[lang] = mo.group(2).strip()
            else:
                name[lang] = t
    kw.update(dd.babel_values('name', **name))
    kw.update(dd.babel_values('abbr', **abbr))
    obj = Concept(**kw)
    yield obj
    if obsoletes is not None:
        if isinstance(obsoletes, dd.Model):
            obsoletes = [obsoletes]
        for obsoleted in obsoletes:
            yield Link(parent=obsoleted, child=obj, type=LinkTypes.obsoletes)

    if jargon is not None:
        if isinstance(jargon, dd.Model):
            jargon = [jargon]
        for domain in jargon:
            yield Link(parent=domain, child=obj, type=LinkTypes.jargon)


def objects():
    nsi = C(
        "NSI (National Statistics Institute)",
        "NIS (Nationales Institut für Statistik)",
        "INS (Institut National de Statistique)",
        "NIS (Nationaal Instituut voor Statistiek)")
    yield nsi
    yield C(
        "Statistics Belgium",
        "GDSWI (Generaldirektion Statistik und Wirtschaftsinformation)",
        "DGSIE (Direction générale Statistique et Information économique)",
        "ADSEI (Algemene Directie Statistiek en Economische Informatie)",
        obsoletes=nsi)
    yield C(
        "NR (National Register)",
        "NR (Nationalregister)",
        "RN (Registre National)",
        "RR (Rijksregister)")
    cpas = C(
        "PCSW (Public Centre for Social Welfare)",
        "ÖSHZ (Öffentliches Sozialhilfezentrum)",
        "CPAS (Centre Public d'Action Sociale)",
        "OCMW (Openbaar Centrum voor Maatschappelijk Welzijn)",
        is_jargon_domain=True)
    yield cpas
    yield C(
        "SPIS (Social Integration Service)",
        "DSBE (Dienst für Sozial-Berufliche Eingliederung)",
        "SISP (Service d'insertion socio-professionnelle)",
        "DSPI (Dienst Socio-Professionele Inschakeling)",
        jargon=cpas)
    yield C(
        "ISIP (Individual social integration project)",
        "VSE (Vertrag zur Sozialen Eingliederung)",
        "PIIS (Projet Individualisé d'Intégration Sociale)",
        jargon=cpas)
    yield C(
        "SSIN (Social Security Identification Number)",
        "INSS (Identifizierungsnummer der Sozialsicherheit)",
        "NISS (N° d'Identification de la Sécurité Sociale)",
        jargon=cpas)
    yield C(
        "Debts consulting",
        "Schuldnerberatung",
        "Médiation de dettes",
        "Schuldbemiddeling ")
    yield C(
        "Social Service",
        "Sozialdienst",
        "Service Social",
        "Sociale dienst")
    yield C(
        "LEA (Local Employment Agency)",
        "LBA (Lokale Beschäftigungsagentur)",
        "ALE (Agence locale pour l'emploi)",
        "PWA (Plaatselijk werkgelegenheidsagentschap)")
    yield C(
        "NEO (National Employment Office)",
        "LfA (Landesamt für Arbeitsbeschaffung)",
        "ONEM (Office national de l'emploi)",
        "RVA (Rijksdienst voor Arbeidsvoorziening)")
    yield C(
        "Head of household",
        "Haushaltsvorstand",
        "Chef de ménage",
        "Haushaltsvorstand")
    yield C(
        "CBSS (Crossroads Bank for Social Security)",
        "ZDSS (Zentrale Datenbank der Sozialen Sicherheit)",
        "BCSS (Banque Carrefour de la Sécurité Sociale)",
        "KSZ (Kruispuntbank van de Sociale Zekerheid)")



from lino.api import rt


def findbyname(model, name):
    """
    Utility function.
    """
    return model.objects.get(name=name)


def objects():
    """This will be called by the :ref:`dpy` deserializer during
    :manage:`prep` and must yield a list of object instances to
    be saved.

    """
    Place = rt.models.lets.Place
    Member = rt.models.lets.Member
    Product = rt.models.lets.Product

    def offer(what, who):
        return rt.models.lets.Offer(
            product=findbyname(Product, what),
            provider=findbyname(Member, who))

    def demand(what, who):
        return rt.models.lets.Demand(
            product=findbyname(Product, what),
            customer=findbyname(Member, who))

    def member(name, place, email=''):
        return rt.models.lets.Member(
            name=name, email=email,
            place=findbyname(Place, place))

    yield Place(name="Tallinn")
    yield Place(name="Tartu")
    yield Place(name="Vigala")
    yield Place(name="Haapsalu")

    yield Product(name="Bread")
    yield Product(name="Buckwheat")
    yield Product(name="Eggs")
    yield Product(name="Sanitary repair work")
    yield Product(name="Building repair work")
    yield Product(name="Electricity repair work")

    yield member("Fred", "Tallinn", 'fred@example.com')
    yield member("Argo", "Haapsalu", 'argo@example.com')
    yield member("Peter", "Vigala", 'peter@example.com')
    yield member("Anne", "Tallinn", 'anne@example.com')
    yield member("Jaanika", "Tallinn", 'jaanika@example.com')

    yield member("Henri", "Tallinn", 'henri@example.com')
    yield member("Mari", "Tartu", 'mari@example.com')
    yield member("Katrin", "Vigala", 'katrin@example.com')

    yield offer("Bread", "Fred")
    yield offer("Buckwheat", "Fred")
    yield offer("Buckwheat", "Anne")
    yield offer("Electricity repair work", "Henri")
    yield offer("Electricity repair work", "Argo")

    yield demand("Buckwheat", "Henri")
    yield demand("Eggs", "Henri")
    yield demand("Eggs", "Mari")




# -*- coding: UTF-8 -*-
# Copyright 2002-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""The :mod:`lino_book` package contains a set of example projects
used both for testing and explaining Lino framework.

It is not published on PyPI because that would make no sense.  You use it by
cloning the repository from GitHub (which is done automatically by
:ref:`getlino`).

.. autosummary::
   :toctree:

   projects

"""

from os.path import join, dirname
from .setup_info import SETUP_INFO
# filename = join(dirname(__file__), 'setup_info.py')
# with open(filename, "rb") as fd: exec(compile(fd.read(), filename, 'exec'))

__version__ = SETUP_INFO['version']
# intersphinx_urls = dict(docs="http://www.lino-framework.org")
srcref_url = 'https://github.com/lino-framework/book/blob/master/%s'

# doc_trees = ['docs']
doc_trees = [ 'docs']
intersphinx_urls = {
    'docs': "http://www.lino-framework.org",
}



# Generated by Django 2.2.4 on 2019-08-09 10:40

from django.db import migrations, models
import django.db.models.deletion
import lino.core.choicelists
import lino.core.fields
import lino.utils.mldbc.fields
import lino_xl.lib.countries.choicelists


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('name', lino.utils.mldbc.fields.BabelCharField(help_text='The designation of this country.', max_length=200, verbose_name='Designation')),
                ('isocode', models.CharField(help_text='The two-letter code for this country as defined by ISO 3166-1.\nFor countries that no longer exist it may be a 4-letter code.', max_length=4, primary_key=True, serialize=False, verbose_name='ISO code')),
                ('short_code', models.CharField(blank=True, help_text='A short abbreviation for regional usage. Obsolete.', max_length=4, verbose_name='Short code')),
                ('iso3', models.CharField(blank=True, help_text='The three-letter code for this country as defined by ISO 3166-1.', max_length=3, verbose_name='ISO-3 code')),
                ('name_de', models.CharField(blank=True, max_length=200, verbose_name='Designation (de)')),
                ('name_fr', models.CharField(blank=True, max_length=200, verbose_name='Designation (fr)')),
            ],
            options={
                'verbose_name': 'Country',
                'verbose_name_plural': 'Countries',
                'ordering': ['isocode'],
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', lino.utils.mldbc.fields.BabelCharField(max_length=200, verbose_name='Designation')),
                ('zip_code', models.CharField(blank=True, max_length=8)),
                ('type', lino.core.choicelists.ChoiceListField(blank=True, choicelist=lino_xl.lib.countries.choicelists.PlaceTypes, choices=[(lino_xl.lib.countries.choicelists.PlaceType('10', 'Member State'), 'Member State'), (lino_xl.lib.countries.choicelists.PlaceType('11', 'Division'), 'Division'), (lino_xl.lib.countries.choicelists.PlaceType('12', 'Region'), 'Region'), (lino_xl.lib.countries.choicelists.PlaceType('13', 'Community'), 'Community'), (lino_xl.lib.countries.choicelists.PlaceType('14', 'Territory'), 'Territory'), (lino_xl.lib.countries.choicelists.PlaceType('20', 'County', 'county'), 'County'), (lino_xl.lib.countries.choicelists.PlaceType('21', 'Province', 'province'), 'Province'), (lino_xl.lib.countries.choicelists.PlaceType('22', 'Shire'), 'Shire'), (lino_xl.lib.countries.choicelists.PlaceType('23', 'Subregion'), 'Subregion'), (lino_xl.lib.countries.choicelists.PlaceType('24', 'Department'), 'Department'), (lino_xl.lib.countries.choicelists.PlaceType('25', 'Arrondissement'), 'Arrondissement'), (lino_xl.lib.countries.choicelists.PlaceType('26', 'Prefecture'), 'Prefecture'), (lino_xl.lib.countries.choicelists.PlaceType('27', 'District', 'district'), 'District'), (lino_xl.lib.countries.choicelists.PlaceType('28', 'Sector'), 'Sector'), (lino_xl.lib.countries.choicelists.PlaceType('50', 'City', 'city'), 'City'), (lino_xl.lib.countries.choicelists.PlaceType('51', 'Town', 'town'), 'Town'), (lino_xl.lib.countries.choicelists.PlaceType('52', 'Municipality', 'municipality'), 'Municipality'), (lino_xl.lib.countries.choicelists.PlaceType('54', 'Parish', 'parish'), 'Parish'), (lino_xl.lib.countries.choicelists.PlaceType('55', 'Township', 'township'), 'Township'), (lino_xl.lib.countries.choicelists.PlaceType('56', 'Quarter', 'quarter'), 'Quarter'), (lino_xl.lib.countries.choicelists.PlaceType('61', 'Borough', 'borough'), 'Borough'), (lino_xl.lib.countries.choicelists.PlaceType('62', 'Small borough', 'smallborough'), 'Small borough'), (lino_xl.lib.countries.choicelists.PlaceType('70', 'Village', 'village'), 'Village')], help_text="The type of this place (whether it's a city. a village, a province...)", max_length=10, verbose_name='Place Type')),
                ('show_type', models.BooleanField(default=False, verbose_name='Show type')),
                ('name_de', models.CharField(blank=True, max_length=200, verbose_name='Designation (de)')),
                ('name_fr', models.CharField(blank=True, max_length=200, verbose_name='Designation (fr)')),
                ('country', models.ForeignKey(help_text='The country this place is in.', on_delete=django.db.models.deletion.PROTECT, to='countries.Country')),
                ('parent', models.ForeignKey(blank=True, help_text='The superordinate geographic place of which this place is a part.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='children', to='countries.Place', verbose_name='Parent')),
            ],
            options={
                'verbose_name': 'Place',
                'verbose_name_plural': 'Places',
                'abstract': False,
                'unique_together': {('country', 'parent', 'name', 'type', 'zip_code')},
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
    ]



# -*- coding: UTF-8 -*-
# Copyright 2017-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Is it allowed to annotate the Sum of a TimeField?
==================================================

Answer: Not under sqlite. See :djangoticket:`19360`.

>>> from lino import startup
>>> startup("lino_book.projects.20121124.settings")
>>> from lino.api.doctest import *

Let's create two tickets:

>>> t1 = Ticket(name="Ticket #1")
>>> t1.save()
>>> t2 = Ticket(name="Ticket #2")
>>> t2.save()
>>> print(Ticket.objects.all())
[<Ticket: Ticket #1>, <Ticket: Ticket #2>]

and some sessions:

>>> Session(ticket=t1,time="0:45",price="0.75").save()
>>> Session(ticket=t1,time="1:30",price="1.50").save()
>>> print(Session.objects.all())
[<Session: at 00:45:00>, <Session: at 01:30:00>]

Get the sum of the prices of all sessions for each ticket:

>>> qs = Ticket.objects.annotate(pricesum=models.Sum('sessions__price'))
>>> print([t.pricesum for t in qs])
[Decimal('2.25'), None]


Now the same with a timefield:

>>> qs = Ticket.objects.annotate(timesum=models.Sum('sessions__time'))
>>> print([str(t.timesum) for t in qs])
['02:15:00','']

The above example raises the following Exception::

  Traceback (most recent call last):
  ...
  TypeError: expected string or buffer

The same exception comes when I use Django development trunk revision 17942.

- `Value conversions of aggregate return values -- is float conversion really required?
  <https://groups.google.com/forum/?fromgroups=#!topic/django-developers/6HQlh2t1j4M>`_

"""

from django.db import models
from django.utils.translation import ugettext_lazy as _
from lino.api import dd


class Ticket(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class Session(models.Model):
    ticket = dd.ForeignKey(Ticket, related_name="sessions")
    time = models.TimeField()
    price = models.DecimalField(max_digits=5, decimal_places=2)

    def __str__(self):
        return "at %s" % self.time



# Generated by Django 2.2.4 on 2019-08-09 10:40

from django.db import migrations, models
import django.db.models.deletion
import lino.core.choicelists
import lino.core.fields
import lino.modlib.system.choicelists
import lino.utils.addressable
import lino.utils.mldbc.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('countries', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CompanyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', lino.utils.mldbc.fields.BabelCharField(max_length=200, verbose_name='Designation')),
                ('abbr', lino.utils.mldbc.fields.BabelCharField(blank=True, max_length=30, verbose_name='Abbreviation')),
                ('abbr_de', models.CharField(blank=True, max_length=30, verbose_name='Abbreviation (de)')),
                ('abbr_fr', models.CharField(blank=True, max_length=30, verbose_name='Abbreviation (fr)')),
                ('name_de', models.CharField(blank=True, max_length=200, verbose_name='Designation (de)')),
                ('name_fr', models.CharField(blank=True, max_length=200, verbose_name='Designation (fr)')),
            ],
            options={
                'verbose_name': 'Organization type',
                'verbose_name_plural': 'Organization types',
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
        migrations.CreateModel(
            name='Partner',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(blank=True, help_text='The primary email address.', max_length=254, verbose_name='e-mail address')),
                ('language', lino.utils.mldbc.fields.LanguageField(blank=True, choices=[('en', 'English'), ('de', 'German'), ('fr', 'French')], help_text='The language to use when communicating with this partner.', max_length=5, verbose_name='Language')),
                ('url', models.URLField(blank=True, verbose_name='URL')),
                ('phone', models.CharField(blank=True, help_text='The primary phone number.', max_length=200, verbose_name='Phone')),
                ('gsm', models.CharField(blank=True, help_text='The primary mobile phone number.', max_length=200, verbose_name='GSM')),
                ('fax', models.CharField(blank=True, max_length=200, verbose_name='Fax')),
                ('zip_code', models.CharField(blank=True, max_length=10, verbose_name='Zip code')),
                ('addr1', models.CharField(blank=True, help_text='Address line before street', max_length=200, verbose_name='Address line before street')),
                ('street_prefix', models.CharField(blank=True, help_text='Text to print before name of street, but to ignore for sorting.', max_length=200, verbose_name='Street prefix')),
                ('street', models.CharField(blank=True, help_text='Name of street, without house number.', max_length=200, verbose_name='Street')),
                ('street_no', models.CharField(blank=True, help_text='House number.', max_length=10, verbose_name='No.')),
                ('street_box', models.CharField(blank=True, help_text='Text to print after street number on the same line.', max_length=10, verbose_name='Box')),
                ('addr2', models.CharField(blank=True, help_text='Address line to print below street line.', max_length=200, verbose_name='Address line after street')),
                ('prefix', models.CharField(blank=True, help_text='An optional name prefix. For organisations this is inserted\nbefore the name, for persons this is inserted between first\nname and last name.', max_length=200, verbose_name='Name prefix')),
                ('name', models.CharField(help_text='The full name of this partner. Used for alphabetic sorting.', max_length=200, verbose_name='Name')),
                ('remarks', models.TextField(blank=True, verbose_name='Remarks')),
                ('city', models.ForeignKey(blank=True, help_text='The locality, i.e. usually a village, city or town.', null=True, on_delete=django.db.models.deletion.PROTECT, to='countries.Place', verbose_name='Locality')),
                ('country', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='countries.Country')),
                ('region', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='contacts_partner_set_by_region', to='countries.Place', verbose_name='County')),
            ],
            options={
                'verbose_name': 'Partner',
                'verbose_name_plural': 'Partners',
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow, lino.utils.addressable.Addressable),
        ),
        migrations.CreateModel(
            name='RoleType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', lino.utils.mldbc.fields.BabelCharField(help_text='A translatable designation. Used e.g. in document templates\nfor contracts.', max_length=200, verbose_name='Designation')),
                ('name_de', models.CharField(blank=True, max_length=200, verbose_name='Designation (de)')),
                ('name_fr', models.CharField(blank=True, max_length=200, verbose_name='Designation (fr)')),
            ],
            options={
                'verbose_name': 'Function',
                'verbose_name_plural': 'Functions',
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow),
        ),
        migrations.CreateModel(
            name='Company',
            fields=[
                ('partner_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='contacts.Partner')),
                ('type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='contacts.CompanyType')),
            ],
            options={
                'verbose_name': 'Organization',
                'verbose_name_plural': 'Organizations',
                'abstract': False,
            },
            bases=('contacts.partner',),
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('partner_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='contacts.Partner')),
                ('title', models.CharField(blank=True, help_text='Used to specify a professional position or academic\nqualification like "Dr." or "PhD".', max_length=200, verbose_name='Title')),
                ('first_name', models.CharField(blank=True, help_text='The first name, also known as given name.', max_length=200, verbose_name='First name')),
                ('middle_name', models.CharField(blank=True, help_text='A space-separated list of all middle names.', max_length=200, verbose_name='Middle name')),
                ('last_name', models.CharField(blank=True, help_text='The last name, also known as family name.', max_length=200, verbose_name='Last name')),
                ('gender', lino.core.choicelists.ChoiceListField(blank=True, choicelist=lino.modlib.system.choicelists.Genders, choices=[(lino.core.choicelists.Choice('M', 'Male', 'male'), 'Male'), (lino.core.choicelists.Choice('F', 'Female', 'female'), 'Female')], help_text='The sex of this person (male or female).', max_length=10, verbose_name='Gender')),
                ('birth_date', lino.core.fields.IncompleteDateField(blank=True, help_text='Uncomplete dates are allowed, e.g. \n"00.00.1980" means "some day in 1980", \n"00.07.1980" means "in July 1980"\nor "23.07.0000" means "on a 23th of July".', verbose_name='Birth date')),
            ],
            options={
                'verbose_name': 'Person',
                'verbose_name_plural': 'Persons',
                'ordering': ['last_name', 'first_name'],
                'abstract': False,
            },
            bases=('contacts.partner', models.Model, lino.core.fields.TableRow),
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.ForeignKey(blank=True, help_text='The function of this person in this organization.', null=True, on_delete=django.db.models.deletion.PROTECT, to='contacts.RoleType')),
                ('company', models.ForeignKey(help_text='The organization where this person has this role.', on_delete=django.db.models.deletion.PROTECT, related_name='rolesbycompany', to='contacts.Company')),
                ('person', models.ForeignKey(help_text='The person having this role in this organization.', on_delete=django.db.models.deletion.PROTECT, related_name='rolesbyperson', to='contacts.Person')),
            ],
            options={
                'verbose_name': 'Contact Person',
                'verbose_name_plural': 'Contact Persons',
                'abstract': False,
            },
            bases=(models.Model, lino.core.fields.TableRow, lino.utils.addressable.Addressable),
        ),
    ]



# Copyright 2014-2015 Rumma & Ko Ltd
# License: BSD, see LICENSE for more details.

"""
Overrides :mod:`lino_xl.lib.contacts` for :mod:`lino.projects.min2`.

.. autosummary::
   :toctree:

   models

"""

from lino_xl.lib.contacts import Plugin


class Plugin(Plugin):

    extends_models = ['Partner', 'Person', 'Company']
    use_vcard_export = True



#!/usr/bin/env python

if __name__ == "__main__":
    import sys
    import os
    os.environ['DJANGO_SETTINGS_MODULE'] = \
        'lino_book.projects.team.settings.demo'
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



# -*- coding: UTF-8 -*-
## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.

from __future__ import unicode_literals

from lino.api import dd, _


class EntryStates(dd.Workflow):
    pass
    
add = EntryStates.add_item
add('10', _("New"), 'new', button_text="☐")
add('20', _("Started"), 'started', button_text="⚒")
add('30', _("Done"), 'done', button_text="☑")
add('40', _("Sleeping"), 'sleeping', button_text="☾")
add('50', _("Cancelled"), 'cancelled', button_text="☒")




# -*- coding: UTF-8 -*-

from lino_book.projects.min2.settings import *


class Site(Site):
    languages = 'en de fr et nl pt-br'

SITE = Site(globals())

SECRET_KEY = "20227"  # see :djangoticket:`20227`



"""
Test whether dump the previous versions can be restored

  $ go lydia
  $ python manage.py test tests.test_restore

"""

from lino.utils.djangotest import RestoreTestCase

class TestCase(RestoreTestCase):
    # tested_versions = ['18.12.0']
    tested_versions = []

    # as long as there is only one production site, testing the migrators
    # would cause increased maintenance effort.




from lino_book.projects.belref.settings import *
SITE = Site(
    globals(), title=Site.verbose_name + " demo",
    default_ui='lino.modlib.bootstrap3')
DEBUG = True
# the following line should always be commented out in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'



from django.db import models
from django.core.exceptions import ValidationError
from lino.api import dd, rt

class Country(dd.Model):
    
    class Meta(object):
        verbose_name = "Country"
        verbose_name_plural = "Countries"

    name = models.CharField(max_length=30)

    def __str__(self):
        return self.name

class City(dd.Model):
    
    class Meta(object):
        verbose_name_plural = "Cities"
        
    country = dd.ForeignKey(Country, on_delete=models.CASCADE)
    name = models.CharField(max_length=30)

    def __str__(self):
        return self.name

class Person(dd.Model):
    
    class Meta(object):
        verbose_name_plural = "Persons"
        
    name = models.CharField(max_length=100)
    birthdate = models.DateField(null=True, blank=True)
    country = dd.ForeignKey(Country, on_delete=models.SET_NULL, null=True)
    city = dd.ForeignKey(City, on_delete=models.SET_NULL, null=True)

    @dd.chooser()
    def city_choices(cls, country):
        return City.objects.filter(country=country)

    def create_city_choice(self, text):
        """
        Called when an unknown city name was given.
        """
        if self.country is None:
            raise ValidationError(
                "Cannot auto-create city %r if country is empty", text)
        return City.lookup_or_create(
            'name', text, country=self.country)

    def __str__(self):
        return self.name
    



# Deprecated. No longer being tested.

from __future__ import print_function

from lino_book.projects.docs.settings import *

EMAIL_TEMPLATE = """\
To: {recipients}
Subject: {subject}
{body}"""


class Site(Site):
    title = "sendchanges example"

    default_user = "robin"
    # user_types_module = None
    # user_types_module = 'lino.modlib.office.roles'

    def send_email(self, subject, sender, body, recipients):
        # override for this test so that it does not actually send
        # anything.
        recipients = ', '.join(recipients)
        print(EMAIL_TEMPLATE.format(**locals()))

    def do_site_startup(self):

        super(Site, self).do_site_startup()

        from lino.utils.sendchanges import subscribe, register
        
        register('contacts.Person', '*',
                 'created_body.eml', 'updated_body.eml')
        e = register('contacts.Partner', 'name',
                     'created_body.eml', 'updated_body.eml')
        e.created_subject = "Created partner {obj}"
        e.updated_subject = "Change in partner {obj}"

        subscribe('john@example.com')
        subscribe('joe@example.com')

SITE = Site(globals())
SITE.user_types_module = None

DEBUG = True



## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.

from django.utils.translation import ugettext_lazy as _
from lino.api import dd, rt
from lino.modlib.notify.actions import NotifyingAction

class StartEntry(dd.ChangeStateAction):
    label = _("Start")
    help_text = _("This action is not allowed when company, body or subject is empty.")
    required_states = 'new cancelled'
    def get_action_permission(self,ar,obj,state):
        # cannot start entries with empty company, subject or body fields
        if not obj.company or not obj.subject or not obj.body:
            return False
        return super(StartEntry,self).get_action_permission(ar, obj, state)


class FinishEntry(StartEntry):
    icon_name='accept'
    label = _("Finish")
    required_states = 'new started'
    help_text = _("Inherts from StartEntry and thus is not allowed when company, body or subject is empty.")
        

class WakeupEntry(dd.ChangeStateAction, NotifyingAction):
    label = _("Wake up")
    required_states = 'sleeping'
    # in our example, waking up an antry will send a notification

    def get_notify_recipients(self, ar, obj):
        for u in rt.models.users.User.objects.all():
            yield (u, u.mail_mode)

    def get_notify_subject(self, ar, obj):
        return _("Entry %s has been reactivated!") % obj
            




from lino.api import ad, _


class Plugin(ad.Plugin):
    "See :class:`lino.core.plugin.Plugin`."

    verbose_name = _("Entries")

    needs_plugins = ['lino.modlib.changes', 'lino.modlib.notify']

    def setup_main_menu(self, site, user_type, m):
        m = m.add_menu(self.app_label, self.verbose_name)
        m.add_action('entries.MyEntries')



# -*- coding: UTF-8 -*-
# Copyright 2014-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""

.. autosummary::
   :toctree:

   doctests
   demo
   www



"""

from lino_noi.lib.noi.settings import *


class Site(Site):

    workflows_module = 'lino_book.projects.migs.workflows'

    # migrations_package = 'lino_book.projects.migs.migrations'

    def get_installed_apps(self):
        # add lino.modlib.restful to the std list of plugins
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.restful'
        # yield 'lino_xl.lib.caldav'
        yield 'lino_xl.lib.mailbox'

    def setup_plugins(self):
        super(Site, self).setup_plugins()
        if self.is_installed('extjs'):
            self.plugins.extjs.configure(enter_submits_form=False)



# -*- coding: UTF-8 -*-
# Copyright 2015 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
from lino.utils.djangotest import WebIndexTestCase

# from lino.utils.djangotest import RemoteAuthTestCase


# class TestCase(RemoteAuthTestCase):

#     def test(self):

#         self.assertEqual(1+1, 2)



from lino.projects.std.settings import *

# configure_plugin('countries', country_code='BE')

class Site(Site):

    verbose_name = "20121124"

    # demo_fixtures = ["few_countries", "few_cities", "demo"]

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino_book.projects.20121124'


SITE = Site(globals())
DEBUG = True


# INSTALLED_APPS = ['lino_book.projects.20121124']
#
# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.sqlite3',
#         'NAME': ':memory:'
#     }
# }


#SECRET_KEY = "123"



# -*- coding: utf-8 -*-
# Copyright 2017-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Tests about how Lino computes whether there are still available
places in a course. That computation can be complex in long-running
courses where participants come and go.

To run just this test::

  $ go roger
  $ python manage.py test tests.test_max_places

"""

from __future__ import unicode_literals

from builtins import str
from lino.api import rt
from lino.api.shell import courses
from lino.api.shell import users
from django.conf import settings

from lino.utils.djangotest import RemoteAuthTestCase
from lino.utils import i2d
from lino.modlib.users.choicelists import UserTypes

from lino.utils.instantiator import create_row as create

# def create(model, **kwargs):
#     obj = model(**kwargs)
#     obj.full_clean()
#     obj.save()
#     return obj
    

class QuickTest(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        Enrolment = rt.models.courses.Enrolment
        Pupil = rt.models.courses.Pupil
        Line = rt.models.courses.Line
        Course = rt.models.courses.Course
        CourseStates = rt.models.courses.CourseStates
        EnrolmentStates = rt.models.courses.EnrolmentStates
        EventType = rt.models.cal.EventType

        
        # room = create(cal.Room, name="First Room")
        lesson = create(EventType, name="Lesson", event_label="Lesson")
        line = create(Line, name="First Line", event_type=lesson)
        obj = create(
            Course,
            line=line,
            # room=room,
            max_places=3,
            #monday=True,
            #start_date=i2d(20140110),
            state=courses.CourseStates.active)
        self.assertEqual(str(obj), "Activity #1")

        # self.assertEqual(settings.SITE.kernel.site, settings.SITE)
        # self.assertEqual(settings.SITE, dd.site)
        # self.assertEqual(settings.SITE.plugins, dd.plugins)
        # self.assertEqual(settings.SITE.plugins.extjs, dd.plugins.extjs)

        settings.SITE.verbose_client_info_message = True
        users.User(username="robin",
                   user_type=UserTypes.admin,
                   language="en").save()
        ses = settings.SITE.login('robin')

        anna = create(Pupil, first_name="Anna")
        bert = create(Pupil, first_name="Bert")
        claire = create(Pupil, first_name="Claire")
        ernie = create(Pupil, first_name="Ernie")

        def ENR(p, start_date, **kwargs):
            return create(
                Enrolment, course=obj, pupil=p,
                state=EnrolmentStates.confirmed,
                start_date=start_date, **kwargs)

        # anna and ernie participated from the beginning.
        # bert stopped in may, and claire started in june.
        # so there were never more than 3 participants.

        # 2015-12-31 : 0 participants, 3 free places
        # 2016-01-01 : 3 participants
        # 2016-05-01 : bert leaves. 2 participants
        # 2016-06-01 : claire starts. 3 participants
        
        ENR(anna, i2d(20160101))
        ENR(bert, i2d(20160101), end_date=i2d(20160501))
        ENR(claire, i2d(20160601))
        ENR(ernie, i2d(20160101))

        self.assertEqual(obj.get_free_places(i2d(20151231)), 3)
        self.assertEqual(obj.get_free_places(i2d(20160101)), 0)
        self.assertEqual(obj.get_free_places(i2d(20160301)), 0)
        self.assertEqual(obj.get_free_places(i2d(20160531)), 1)
        self.assertEqual(obj.get_free_places(i2d(20161231)), 0)
        
        #self.assertEqual(enr.get_confirm_veto(None), '')

        # 20180731 the default value for the enrolment state was a
        # string which became an EnrolmentStates choice only during
        # full_clean().  Now this case is being resolved in
        # ChoiceListField.__init__().
        
        enr = Enrolment(course=obj, pupil=anna)
        self.assertEqual(enr.state, EnrolmentStates.requested)



from lino.projects.std.settings import *


class Site(Site):

    title = "Cool Polls"
    project_name = "My First Polls"

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        # yield 'polls'
        yield 'lino_book.projects.polls.polls'

    def setup_menu(self, user_type, main):
        super(Site, self).setup_menu(user_type, main)
        m = main.add_menu("polls", "Polls")
        m.add_action('polls.Questions')
        m.add_action('polls.Choices')

SITE = Site(globals())

# your local settings here

DEBUG = True



from lino.projects.std.settings import *

class Site(Site):

    title = "MLDBC Tutorial"

    demo_fixtures = ['demo']

    languages = 'en fr'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        # yield 'lino.modlib.system'
        yield 'lino_book.projects.mldbc'

    def setup_menu(self, user_type, main):
        m = main.add_menu("products", "Products")
        m.add_action('mldbc.Products')
        super(Site, self).setup_menu(user_type, main)
        
SITE = Site(globals())

DEBUG = True



# -*- coding: utf-8 -*-
# Copyright 2008-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
This module contains some relatively quick tests that don't load any
fixtures.

To run only this test::

  $ go min1
  $ python manage.py test

"""

from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import translation
from atelier.test import TestCase
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core.management import call_command
import subprocess, os
import unittest

from lino.api import dd, rt
from lino.modlib.users.choicelists import UserTypes
from lino.utils.djangotest import RemoteAuthTestCase
from lino.utils.instantiator import create_and_get
from lino_xl.lib.contacts import models as contacts

Genders = dd.Genders

CYPRESS = "../../../node_modules/cypress/bin/cypress"

class QuickTest(RemoteAuthTestCase):

    def test01(self):
        """
        Tests some basic funtionality.
        """
        self.assertEqual(
            settings.MIDDLEWARE, (
                'django.middleware.common.CommonMiddleware',
                'django.middleware.locale.LocaleMiddleware',
                'django.contrib.sessions.middleware.SessionMiddleware',
                'lino.core.auth.middleware.AuthenticationMiddleware',
                'lino.core.auth.middleware.WithUserMiddleware',
                'lino.core.auth.middleware.DeviceTypeMiddleware',
                'lino.core.auth.middleware.RemoteUserMiddleware',
                'lino.utils.ajax.AjaxExceptionResponse'))
            # settings.MIDDLEWARE_CLASSES, (
            #     'django.middleware.common.CommonMiddleware',
            #     'django.middleware.locale.LocaleMiddleware',
            #     'lino.core.auth.RemoteUserMiddleware',
            #     'lino.utils.ajax.AjaxExceptionResponse'))

        Person = rt.models.contacts.Person
        Partner = rt.models.contacts.Partner
        Country = rt.models.countries.Country
        Place = rt.models.countries.Place
        PlaceTypes = rt.models.countries.PlaceTypes
        
        ee = create_and_get(Country,
                            isocode='EE', **dd.babelkw('name',
                                                    de="Estland",
                                                    fr='Estonie',
                                                    en="Estonia",
                                                    nl='Estland',
                                                    et='Eesti',
                                                    ))
        be = create_and_get(Country,
                            isocode='BE', **dd.babelkw('name',
                                                    de="Belgien",
                                                    fr='Belgique',
                                                    en="Belgium",
                                                    nl='Belgie',
                                                    et='Belgia',
                                                    ))

        eupen = create_and_get(
            Place, name=u'Eupen', country=be, zip_code='4700')

        vigala = create_and_get(Place,
                                name='Vigala',
                                country=ee,
                                type=PlaceTypes.municipality)

        luc = create_and_get(Person,
                             first_name='Luc', last_name='Saffre',
                             gender=Genders.male,
                             country=ee, street='Uus', street_no='1',
                             addr2=u'Vana-Vigala küla',
                             city=vigala, zip_code='78003')

        settings.SITE.uppercase_last_name = True

        """If the following tests raise a "DoesNotExist: Company matching
        query does not exist" then this may come because
        Site._site_config has been filled before the database switched
        from the real db to test db.  and not properly reset.

        """

        with translation.override('en'):
            self.assertEquals(luc.address, u'''\
Mr Luc SAFFRE
Uus 1
Vana-Vigala küla
78003 Vigala vald
Estonia''')

        with translation.override('de'):
            self.assertEquals(luc.address, u'''\
Herrn Luc SAFFRE
Uus 1
Vana-Vigala küla
78003 Vigala vald
Estland''')
            self.assertEquals(luc.address_html, '''\
<p>Herrn Luc SAFFRE<br/>Uus 1<br/>Vana-Vigala küla<br/>78003 Vigala vald<br/>Estland</p>''')

        # "new" or "full" style is when the database knows the
        # geographic hierarchy. We then just select "Vana-Vigala" as
        # the "City".

        vana_vigala = create_and_get(Place,
                                     name='Vana-Vigala',
                                     country=ee,
                                     parent=vigala,
                                     type=PlaceTypes.village,
                                     zip_code='78003')

        meeli = create_and_get(Person,
                               first_name='Meeli', last_name='Mets',
                               gender=Genders.female,
                               country=ee, street='Hirvepargi',
                               street_no='123',
                               city=vana_vigala)

        with translation.override('en'):
            self.assertEquals(meeli.address, u'''\
Mrs Meeli METS
Hirvepargi 123
Vana-Vigala küla
78003 Vigala vald
Estonia''')

        root = create_and_get(settings.SITE.user_model,
                              username='root', language='',
                              user_type=UserTypes.admin)

        """
        disable SITE.is_imported_partner() otherwise 
        disabled_fields may contain more than just the 'id' field.
        """
        save_iip = settings.SITE.is_imported_partner

        def f(obj):
            return False
        settings.SITE.is_imported_partner = f

        """
        Note that we must specify the language both in the user 
        and in HTTP_ACCEPT_LANGUAGE because...
        """

        luc = Person.objects.get(name__exact="Saffre Luc")
        self.assertEqual(luc.pk, contacts.PARTNER_NUMBERS_START_AT)

        url = settings.SITE.buildurl(
            'api', 'contacts', 'Person',
            '%d?query=&an=detail&fmt=json' % luc.pk)
        #~ url = '/api/contacts/Person/%d?query=&an=detail&fmt=json' % luc.pk
        if settings.SITE.get_language_info('en'):
            root.language = 'en'
            root.save()
            self.client.force_login(root)
            response = self.client.get(
                url, REMOTE_USER='root', HTTP_ACCEPT_LANGUAGE='en')
            result = self.check_json_result(
                response, 'data disable_delete id navinfo param_values title')
            self.assertEqual(result['data']['country'], "Estonia")
            self.assertEqual(result['data']['gender'], "Male")

        if settings.SITE.get_language_info('de'):
            root.language = 'de'
            root.save()
            response = self.client.get(
                url, REMOTE_USER='root', HTTP_ACCEPT_LANGUAGE='de')
            result = self.check_json_result(
                response,
                'navinfo disable_delete data id param_values title')
            self.assertEqual(result['data']['country'], "Estland")
            self.assertEqual(result['data']['gender'], "Männlich")
            #~ self.assertEqual(result['data']['disabled_fields'],['contact_ptr_id','id'])
            #~ self.assertEqual(result['data']['disabled_fields'],['id'])
            df = result['data']['disabled_fields']
            self.assertEqual(df['id'], True)

        if settings.SITE.get_language_info('fr'):
            root.language = 'fr'
            root.save()
            response = self.client.get(
                url, REMOTE_USER='root', HTTP_ACCEPT_LANGUAGE='fr')
            result = self.check_json_result(
                response, 'navinfo disable_delete data id param_values title')
            self.assertEqual(result['data']['country'], "Estonie")
            self.assertEqual(result['data']['gender'], u"Masculin")

        #~ root.language = lang
        #~ root.save()
        # restore is_imported_partner method
        settings.SITE.is_imported_partner = save_iip

        #~ def test03(self):
        """
        Test the following situation:
        
        - User 1 opens the :menuselection:`Configure --> System--> System Parameters` dialog
        - User 2 creates a new Person (which increases next_partner_id)
        - User 1 clicks on `Save`.
        
        `next_partner_id` may not get overwritten 
        
        """
        # User 1
        sc = settings.SITE.site_config
        self.assertEqual(sc.next_partner_id,
                         contacts.PARTNER_NUMBERS_START_AT + 2)
        sc.update(next_partner_id=12345)
        
        # SiteConfigs = settings.SITE.models.system.SiteConfigs
        # elem = SiteConfigs.get_row_by_pk(None, settings.SITE.config_id)
        # self.assertEqual(elem.next_partner_id,
        #                  contacts.PARTNER_NUMBERS_START_AT + 2)
        # elem.next_partner_id = 12345
        # elem.full_clean()
        # elem.save()
        #~ print "saved"
        sc = settings.SITE.site_config  # re-read it from db
        self.assertEqual(sc.next_partner_id, 12345)
        john = create_and_get(Partner, name='John')
        self.assertEqual(john.pk, 12345)
        self.assertEqual(sc.next_partner_id, 12346)
        sc = settings.SITE.site_config  # re-read it from db
        self.assertEqual(sc.next_partner_id, 12346)

    def unused_test03(self):
        """
        Test the following situation:

        - User 1 opens the :menuselection:`Configure --> System--> System Parameters` dialog
        - User 2 creates a new Person (which increases next_partner_id)
        - User 1 clicks on `Save`.

        `next_partner_id` may not get overwritten

        """

        url = settings.SITE.buildurl(
            'api', 'system', 'SiteConfigs', '1?an=detail&fmt=json')
        response = self.client.get(url, REMOTE_USER='root')
        result = self.check_json_result(
            response, 'navinfo disable_delete data id title')
        """
        `test01` created one Person, so next_partner_id should be at 101:
        """
        data = result['data']
        self.assertEqual(data['next_partner_id'],
                         contacts.PARTNER_NUMBERS_START_AT + 1)

        data['next_partner_id'] = 12345
        #~ pprint(data)
        response = self.client.put(
            url, data,
            #~ content_type="application/x-www-form-urlencoded; charset=UTF-8",
            REMOTE_USER='root')

        result = self.check_json_result(
            response, 'message rows success data_record')
        data = result['data_record']['data']

        john = create_and_get(Person, first_name='John', last_name='Smith')
        # fails: self.assertEqual(john.pk,12345)

    def test02(self):
        # This case demonstrates that ordering does not ignore case, at
        # least in sqlite. we would prefer to have `['adams', 'Zybulka']`,
        # but we get `['Zybulka', 'adams']`.

        contacts = rt.models.contacts
        contacts.Partner(name="Zybulka").save()
        contacts.Partner(name="adams").save()
        ar = rt.login().spawn(contacts.Partners)
        l = [p.name for p in ar]
        expected = ['Zybulka', 'adams']
        self.assertEqual(l, expected)

    def test03(self):
        User = rt.models.users.User
        foo = User(last_name="Foo")
        try:
            foo.full_clean()
            self.fail("Expected ValidationError")
        except ValidationError:
            pass

@unittest.skipIf(not os.path.exists(CYPRESS),
                     "Cypress is not installed")
class CypressTest(StaticLiveServerTestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        call_command('prep', interactive=False, verbosity=0)

    def test_cypress(self):
        print(self.live_server_url)
        os.environ.update({
            #'CYPRESS_HOST':self.live_server_url,
            #'cypress_api_server':self.live_server_url,
            'cypress_baseUrl':self.live_server_url
        })
        command = '{} run -C ../../../cypress.json -s cypress/integration/min1/* --config video=false'.format(CYPRESS,self.live_server_url)
        print(command)
        p = subprocess.Popen(command.split())
        out, err = p.communicate()


## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.

from django.db import models
from lino.api import dd
from django.utils.translation import ugettext_lazy as _
from lino.mixins import CreatedModified
from lino.modlib.users.mixins import My, UserAuthored

from .choicelists import EntryStates

class Entry(CreatedModified, UserAuthored):
    
    workflow_state_field = 'state'
    
    class Meta:
        verbose_name = _("Entry")
        verbose_name_plural = _("Entries")
        
    
    subject = models.CharField(_("Subject"),blank=True,max_length=200)
    body = dd.RichTextField(_("Body"),blank=True)
    company = dd.ForeignKey('contacts.Company',blank=True,null=True)
    state = EntryStates.field(blank=True, default='new')
    
class Entries(dd.Table):
    model = Entry    
    detail_layout = """
    id user created modified
    subject
    company workflow_buttons
    body
    """
    insert_layout = """
    company
    subject
    """
    
class EntriesByCompany(Entries):
    master_key = 'company'
    column_names = "modified user subject workflow_buttons *"
    
class MyEntries(My, Entries):
    column_names = "modified subject workflow_buttons *"




## Copyright 2013-2018 Rumma & Ko Ltd
## This file is part of the Lino project.


from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from lino.api import dd
from lino import mixins
from lino.modlib.users.mixins import UserAuthored

# contacts = dd.resolve_app('contacts')


class Company(dd.Model):
    class Meta:
        verbose_name = _("Company")
        verbose_name_plural = _("Companies")

    name = models.CharField(_("Name"), blank=True, max_length=200)
    street = models.CharField(_("Street"), blank=True, max_length=200)
    city = models.CharField(_("City"), blank=True, max_length=200)
    
    def __str__(self):
        return self.name

class EntryType(mixins.BabelDesignated):
    class Meta:
        verbose_name = _("Entry Type")
        verbose_name_plural = _("Entry Types")
        
    
class EntryTypes(dd.Table):
    model = EntryType
    
    
class Entry(UserAuthored):
    
    class Meta:
        verbose_name = _("Entry")
        verbose_name_plural = _("Entries")
        
    date = models.DateField(_("Date"))
    entry_type = dd.ForeignKey(EntryType)
    subject = models.CharField(_("Subject"), blank=True, max_length=200)
    body = dd.RichTextField(_("Body"), blank=True)
    company = dd.ForeignKey(Company)


class Entries(dd.Table):
    model = Entry
    detail_layout = """
    id user date company
    subject
    body
    """
    insert_layout = """
    user date company
    subject
    """
    parameters = mixins.ObservedDateRange(
        entry_type=dd.ForeignKey(
            EntryType, blank=True, null=True,
            help_text=_("Show only entries of this type.")),
        company=dd.ForeignKey(Company,
            blank=True, null=True,
            help_text=_("Show only entries of this company.")),
        user=dd.ForeignKey(
            settings.SITE.user_model,
            blank=True, null=True,
            help_text=_("Show only entries by this user.")))
    params_layout = """
    user start_date end_date
    company entry_type
    """
    
    @classmethod
    def get_request_queryset(cls, ar):
        qs = super(Entries, cls).get_request_queryset(ar)
        if ar.param_values.end_date:
            qs = qs.filter(date__lte=ar.param_values.end_date)
        if ar.param_values.start_date:
            qs = qs.filter(date__gte=ar.param_values.start_date)
        if ar.param_values.user:
            qs = qs.filter(user=ar.param_values.user)
        if ar.param_values.entry_type:
            qs = qs.filter(entry_type=ar.param_values.entry_type)
        if ar.param_values.company:
            qs = qs.filter(company=ar.param_values.company)
        return qs
    
    @classmethod
    def param_defaults(cls, ar, **kw):
        kw = super(Entries, cls).param_defaults(ar, **kw)
        kw.update(user=ar.get_user())
        return kw
        

class EntriesByCompany(Entries):
    master_key = 'company'

class CompanyDetail(dd.DetailLayout):
    main = """
    name
    street city
    EntriesByCompany
    """

class Companies(dd.Table):
    model = Company
    detail_layout = CompanyDetail()

class CompaniesWithEntryTypes(dd.VentilatingTable, Companies):
    label = _("Companies with Entry Types")
    hide_zero_rows = True
    parameters = mixins.ObservedDateRange()
    params_layout = "start_date end_date"
    editable = False
    auto_fit_column_widths = True
    
    @classmethod
    def param_defaults(cls, ar, **kw):
        kw = super(CompaniesWithEntryTypes, cls).param_defaults(ar, **kw)
        kw.update(end_date=settings.SITE.today())
        return kw

    @classmethod
    def get_ventilated_columns(self):
        def w(et):
            # return a getter function for a RequestField on the given
            # EntryType.

            def func(fld, obj, ar):
                #~ mi = ar.master_instance
                #~ if mi is None: return None
                pv = dict(
                    start_date=ar.param_values.start_date,
                    end_date=ar.param_values.end_date)
                if et is not None:
                    pv.update(entry_type=et)
                pv.update(company=obj, user=ar.get_user())
                return Entries.request(param_values=pv)
            return func
        for et in EntryType.objects.all():
            yield dd.RequestField(w(et), verbose_name=str(et))
        yield dd.RequestField(w(None), verbose_name=_("Total"))
    
   
@dd.receiver(dd.post_save, sender=EntryType)
def my_setup_columns(sender, **kw):
    CompaniesWithEntryTypes.setup_columns()


# @dd.receiver(dd.post_startup)
# def my_details_setup(sender, **kw):
#     self = sender
#     self.models.contacts.Companies.add_detail_tab(
#         'entries', 'watch2.EntriesByCompany')





from lino.api import rt
from lino.utils.cycler import Cycler

# thanks to http://fantasynamegenerators.com/restaurant-names.php#.VM1yRWNgtx0
RESTAURANT_NAMES = """
The Chopping Shack
The Abacus Well
The Olive Lounge
The Autumn Bite
The Private Mission
Nova
Babylon
Blossoms
Whisperwind
Catch
"""


def objects():
    Person = rt.models.app.Person
    Restaurant = rt.models.app.Restaurant
    Place = rt.models.app.Place

    anne = Person(name="Anne")
    yield anne

    bert = Person(name="Bert")
    yield bert

    claude = Person(name="Claude")
    yield claude

    dirk = Person(name="Dirk")
    yield dirk

    ernie = Person(name="Ernie")
    yield ernie

    fred = Person(name="Fred")
    yield fred

    PERSONS = Cycler(Person.objects.all())

    p = Place(name="Bert's pub", ceo=PERSONS.pop())
    yield p
    p.owners.add(anne)
    p.owners.add(bert)

    for name in RESTAURANT_NAMES.strip().splitlines():
        p = Place(name=name, ceo=PERSONS.pop())
        yield p
        r = Restaurant(place=p)
        yield r
        p.owners.add(PERSONS.pop())
        r.cooks.add(PERSONS.pop())




"""
The test suite for this demo project.

It is being run as part of the :ref:`book` test suite.  You can
manually run just this suite using::

  $ go adg
  $ python manage.py test


.. autosummary::
   :toctree:

   test_beid
   test_nullchar
   test_simple
   test_webindex
"""



import datetime

from ..settings import *


class Site(Site):
    is_demo_site = True
    # ignore_dates_after = datetime.date(2019, 05, 22)
    the_demo_date = datetime.date(2017,3,12)
    
    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino_xl.lib.b2c'
    
SITE = Site(globals())
DEBUG = True
SITE.plugins.ledger.configure(start_year=2016)



from django.db import models


class Addressable(models.Model):
    class Meta:
        abstract = True
    street = models.CharField(max_length=255, blank=True)
        

class Restaurant(Addressable):
    class Meta:
        abstract = True

    # restaurant_addressable = models.OneToOneField(
    #     Addressable, parent_link=True, on_delete=models.CASCADE)
    
    name = models.CharField(max_length=255)
    

class Bar(Restaurant):
    class Meta:
        abstract = True

    # bar_restaurant = models.OneToOneField(
    #     Restaurant, parent_link=True, on_delete=models.CASCADE)
    
    min_age = models.IntegerField()
    

class Pizzeria(Restaurant):
    
    # pizzeria_restaurant = models.OneToOneField(
    #     Restaurant, parent_link=True, on_delete=models.CASCADE)

    specialty = models.CharField(max_length=255)

    
class PizzeriaBar(Bar, Pizzeria):
    pizza_bar_specific_field = models.CharField(max_length=255)

    # pizzeriabar_pizzeria = models.OneToOneField(
    #     Pizzeria, parent_link=True, on_delete=models.CASCADE)

    # pizzeriabar_bar = models.OneToOneField(
    #     Bar, parent_link=True, on_delete=models.CASCADE)




from lino.api import dd
from .models import Person, City, Country

class Persons(dd.Table):
    model = Person
    detail_layout = dd.DetailLayout("""
    name
    country
    city
    """, window_size=(50, 'auto'))

    insert_layout = """
    name
    country
    city
    """


class Cities(dd.Table):
    model = City

class Countries(dd.Table):
    model = Country



from lino.projects.std.settings import *
SITE = Site(
    globals(),
    'lino_book.projects.actors',
    # user_model=None,
    demo_fixtures=['demo'])
#print(SITE.is_local_project_dir)
#SITE.is_local_project_dir = False
#from lino.core.utils import is_devserver
#print(is_devserver())
# print(SITE.installed_plugins)
# print(MEDIA_URL, MEDIA_ROOT)



# -*- coding: utf-8 -*-
# Copyright 2013-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Tests about generating automatic events of a course.  Look at the
source code!

To run just this test::

  $ cd go roger
  $ python manage.py test

"""

from __future__ import unicode_literals

from builtins import str
from lino.api.shell import cal
from lino.api.shell import courses
from lino.api.shell import users
from django.conf import settings

from lino.utils.djangotest import RemoteAuthTestCase
from lino.utils import i2d
from lino.modlib.users.choicelists import UserTypes


def create(model, **kwargs):
    obj = model(**kwargs)
    obj.full_clean()
    obj.save()
    return obj
    

class QuickTest(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        # Create a room, event type, series and a course
        
        room = create(cal.Room, name="First Room")
        lesson = create(cal.EventType, name="Lesson", event_label="Lesson")
        line = create(courses.Line, name="First Line", event_type=lesson)
        obj = create(
            courses.Course,
            line=line,
            room=room,
            max_events=5,
            monday=True,
            state=courses.CourseStates.active,
            start_date=i2d(20140110))
        self.assertEqual(str(obj), "Activity #1")

        # self.assertEqual(settings.SITE.kernel.site, settings.SITE)
        # self.assertEqual(settings.SITE, dd.site)
        # self.assertEqual(settings.SITE.plugins, dd.plugins)
        # self.assertEqual(settings.SITE.plugins.extjs, dd.plugins.extjs)

        settings.SITE.verbose_client_info_message = True
        users.User(username="robin",
                   user_type=UserTypes.admin,
                   language="en").save()
        ses = settings.SITE.login('robin')

        # utility function which runs update_events and checks whether
        # info_message and output of cal.EntriesByController are as
        # expected:        
        def check_update(obj, msg1, msg2):
            res = ses.run(obj.do_update_events)
            self.assertEqual(res['success'], True)
            print(res['info_message'])
            self.assertEqual(res['info_message'].strip(), msg1.strip())
            ar = ses.spawn(cal.EntriesByController, master_instance=obj)
            s = ar.to_rst(column_names="when_text state summary", nosummary=True)
            # print(s)
            self.assertEqual(s.strip(), msg2.strip())
            
        # Run do_update_events a first time
        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-13 and 2020-05-22 (max. 5).
Update presences for Activity #1 Lesson 1 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 2 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 3 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 4 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 5 : 0 created, 0 unchanged, 0 deleted.
5 row(s) have been updated.
""", """
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 13/01/2014   Suggested   Lesson 1
 Mon 20/01/2014   Suggested   Lesson 2
 Mon 27/01/2014   Suggested   Lesson 3
 Mon 03/02/2014   Suggested   Lesson 4
 Mon 10/02/2014   Suggested   Lesson 5
================ =========== ===================
""")
        
        # Decrease max_events and check whether the superfluous events
        # get removed.
        
        obj.max_events = 3
        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-13 and 2020-05-22 (max. 3).
2 row(s) have been updated.""", """
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 13/01/2014   Suggested   Lesson 1
 Mon 20/01/2014   Suggested   Lesson 2
 Mon 27/01/2014   Suggested   Lesson 3
================ =========== ===================
""")
        
        # Run do_update_events for 5 events a second time
        obj.max_events = 5
        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-13 and 2020-05-22 (max. 5).
Update presences for Activity #1 Lesson 4 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 5 : 0 created, 0 unchanged, 0 deleted.
2 row(s) have been updated.""", """
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 13/01/2014   Suggested   Lesson 1
 Mon 20/01/2014   Suggested   Lesson 2
 Mon 27/01/2014   Suggested   Lesson 3
 Mon 03/02/2014   Suggested   Lesson 4
 Mon 10/02/2014   Suggested   Lesson 5
================ =========== ===================
""")

        
        # Now we want to skip the 2nd event.  We click on "Move next"
        # on this event. Lino then moves all subsequent events
        # accordingly.

        ar = cal.EntriesByController.request(
            master_instance=obj,
            known_values=dict(
                start_date=i2d(20140120)))
        e = ar.data_iterator[0]
        self.assertEqual(e.state, cal.EntryStates.suggested)
        #
        res = ses.run(e.move_next)

        self.assertEqual(res['success'], True)
        expected = """\
Move down for Activity #1 Lesson 2...
Generating events between 2014-01-13 and 2020-05-22 (max. 5).
Lesson 2 has been moved from 2014-01-20 to 2014-01-27.
1 row(s) have been updated."""
        self.assertEqual(res['info_message'], expected)

        # The event is now in state "draft" because it has been
        # modified by the user.

        self.assertEqual(e.state, cal.EntryStates.draft)
        # e.full_clean()
        # e.save()

        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-13 and 2020-05-22 (max. 5).
Lesson 2 has been moved from 2014-01-20 to 2014-01-27.
0 row(s) have been updated.
""","""
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 13/01/2014   Suggested   Lesson 1
 Mon 27/01/2014   Draft       Lesson 2
 Mon 03/02/2014   Suggested   Lesson 3
 Mon 10/02/2014   Suggested   Lesson 4
 Mon 17/02/2014   Suggested   Lesson 5
================ =========== ===================
""")

        # Now we imagine that February 3 is the National Day in our
        # country and that we create the rule for this only now.  So
        # we have a conflict because Lino created an appointment on
        # that date. Of course the National Day must *not* move to an
        # alternative date.

        et = create(cal.EventType, name="Holiday", all_rooms=True)
        national_day = create(
            cal.RecurrentEvent,
            name="National Day", event_type=et,
            start_date=i2d(20140203),
            every_unit=cal.Recurrencies.yearly)

        res = ses.run(national_day.do_update_events)
        self.assertEqual(res['success'], True)
        expected = """\
Update Events for National Day...
Generating events between 2014-02-03 and 2020-05-22 (max. 72).
Reached upper date limit 2020-05-22
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
Update presences for Recurring event #1 National Day : 0 created, 0 unchanged, 0 deleted.
7 row(s) have been updated."""
        self.assertEqual(res['info_message'], expected)
        ar = ses.spawn(
            cal.EntriesByController, master_instance=national_day)
        s = ar.to_rst(column_names="when_text state", nosummary=True)
        # print s
        self.assertEqual(s, """\
================ ===========
 When             State
---------------- -----------
 Mon 03/02/2014   Suggested
 Tue 03/02/2015   Suggested
 Wed 03/02/2016   Suggested
 Fri 03/02/2017   Suggested
 Sat 03/02/2018   Suggested
 Sun 03/02/2019   Suggested
 Mon 03/02/2020   Suggested
================ ===========

""")

        # the national day is now conflicting with our Lesson 3:
        ce = ar[0]
        self.assertEqual(ce.summary, "National Day")
        self.assertEqual(ce.start_date.year, 2014)
        ar = ses.spawn(
            cal.ConflictingEvents, master_instance=ce)
        s = ar.to_rst(column_names="when_text state auto_type")
        # print s
        self.assertEqual(s, """\
==================== =========== =======
 When                 State       No.
-------------------- ----------- -------
 Mon 03/02/2014       Suggested   3
 **Total (1 rows)**               **3**
==================== =========== =======

""")

        # delete all lessons and start again with a virgin series

        cal.Event.objects.filter(event_type=lesson).delete()

        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-13 and 2020-05-22 (max. 5).
Lesson 4 wants 2014-02-03 but conflicts with <QuerySet [Event #8 ('Recurring event #1 National Day')]>, moving to 2014-02-10. 
Update presences for Activity #1 Lesson 1 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 2 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 3 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 4 : 0 created, 0 unchanged, 0 deleted.
Update presences for Activity #1 Lesson 5 : 0 created, 0 unchanged, 0 deleted.
5 row(s) have been updated.
""", """
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 13/01/2014   Suggested   Lesson 1
 Mon 20/01/2014   Suggested   Lesson 2
 Mon 27/01/2014   Suggested   Lesson 3
 Mon 10/02/2014   Suggested   Lesson 4
 Mon 17/02/2014   Suggested   Lesson 5
================ =========== ===================
""")

        # we move the first lesson one week down and check whether
        # remaining entries get adapted. We manually set the state to
        # draft (this is automatically done when using the web ui).
        
        e = cal.Event.objects.get(event_type=lesson, auto_type=1)
        e.start_date = i2d(20140120)
        e.state = cal.EntryStates.draft
        e.full_clean()
        e.save()

        check_update(obj, """
Update Events for Activity #1...
Generating events between 2014-01-27 and 2020-05-22 (max. 5).
Lesson 3 wants 2014-02-03 but conflicts with <QuerySet [Event #8 ('Recurring event #1 National Day')]>, moving to 2014-02-10. 
0 row(s) have been updated.
        """, """
================ =========== ===================
 When             State       Short description
---------------- ----------- -------------------
 Mon 20/01/2014   Draft       Lesson 1
 Mon 27/01/2014   Suggested   Lesson 2
 Mon 10/02/2014   Suggested   Lesson 3
 Mon 17/02/2014   Suggested   Lesson 4
 Mon 24/02/2014   Suggested   Lesson 5
================ =========== ===================
""")

        # we cancel the third lesson and see whether Lino adds a 

        e = cal.Event.objects.get(event_type=lesson, auto_type=3)
        e.state = cal.EntryStates.cancelled
        e.auto_type = None
        e.full_clean()
        e.save()

        # check_update(obj, "", "")



from django.db import models
from lino.api import dd


class PartnerType(dd.Model):
    name = models.CharField("Name", max_length=20)
    
    def __str__(self):
        return self.name


class Partner(dd.Model):

    class Meta:
        verbose_name = "Partner"
        verbose_name_plural = "Partners"

    type = dd.ForeignKey(PartnerType)
    name = models.CharField("Name", max_length=30)


class Person(Partner):
    first_name = models.CharField("First name", max_length=20)
    last_name = models.CharField("Last name", max_length=20)

    class Meta:
        verbose_name = "Person"
        verbose_name_plural = "Persons"





# -*- coding: UTF-8 -*-
# Copyright 2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Fixtures specific for Lino Care.

.. autosummary::
   :toctree:

   tickets


"""

from lino_xl.lib.tickets import *



# -*- coding: utf-8 -*-
# Copyright 2013-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Runs some tests about the Sequenced mixin.

You can run only these tests by issuing::

  $ go apc
  $ python manage.py test tests.test_sequenced

"""

from __future__ import unicode_literals
from __future__ import print_function
from builtins import str

from django.core.exceptions import ValidationError
from lino.utils.djangotest import RemoteAuthTestCase
from lino.api import rt
from lino.utils.mti import delete_child
from lino.utils.instantiator import create_row


class Tests(RemoteAuthTestCase):
    maxDiff = None

    def test01(self):
        from lino.modlib.users.choicelists import UserTypes
        User = rt.models.users.User
        Account = rt.models.ledger.Account
        
        Journal = rt.models.ledger.Journal
        Partner = rt.models.contacts.Partner
        Person = rt.models.contacts.Person
        Company = rt.models.contacts.Company
        Role = rt.models.contacts.Role
        # Account = rt.models.sepa.Account
        Invoice = rt.models.vat.VatAccountInvoice
        VoucherTypes = rt.models.ledger.VoucherTypes
        JournalGroups = rt.models.ledger.JournalGroups

        robin = create_row(
            User, username='robin',
            user_type=UserTypes.admin, language="en")
        ar = rt.login('robin')
        
        a = create_row(Account, name="A")
        b = create_row(Account, name="B")
        c = create_row(Account, name="C")
        d = create_row(Account, name="D")

        lst = [i.seqno for i in Account.objects.order_by('name')]
        self.assertEqual(lst, [1, 2, 3, 4])
        
        self.assertEqual(a.seqno, 1)
        self.assertEqual(b.seqno, 2)
        self.assertEqual(c.seqno, 3)
        self.assertEqual(d.seqno, 4)

        response = a.move_down()
        
        self.assertEqual(response, {
            'message': 'Renumbered 1 of 3 siblings.',
            'success': True,
            'refresh_all': True})

        # NOTE that a, b, c and d are invalid now
        a = b = c = d = None

        lst = [i.seqno for i in Account.objects.order_by('name')]
        self.assertEqual(lst, [2, 1, 3, 4])
        
        lst = [i.name for i in Account.objects.order_by('seqno')]
        self.assertEqual(''.join(lst), "BACD")
        
        # fr = create_row(Country, isocode="FR")




# from lino_book.projects.min2.settings import *
from ..settings import *
from lino.utils import i2d

class Site(Site):
    languages = "en de fr"
    the_demo_date = i2d(20170819)

SITE = Site(globals())
DEBUG = True



# Copyright 2008-2014 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Default settings for :doc:`/igen/index`.

"""
import os
import sys
from os.path import join, dirname, normpath, abspath
from lino.projects.std.settings import *


class Site(Site):

    languages = ['en']

    title = "Lino/iGen"
    domain = "igen-demo.saffre-rumma.net"
    help_url = "http://lino.saffre-rumma.net/igen/index.html"

    #~ residence_permit_upload_type = None
    #~ work_permit_upload_type = None
    #~ driving_licence_upload_type = None
    # ledger_providers='4400',
    # ledger_customers='4000',
    # ~ sales_base_account = None # '7000',
    # ~ sales_vat_account = None # '4510',

    #~ def init_site_config(self,sc):
        #~ super(IgenSite,self).init_site_config(sc)
        #~ sc.next_partner_id = 200000

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.gfks'
        yield 'lino.modlib.system'
        yield 'lino.modlib.users'
        yield 'lino_xl.lib.countries'

        yield 'lino_xl.lib.contacts'
        #~ 'lino_xl.lib.notes'
        yield 'lino_xl.lib.extensible'
        yield 'lino_xl.lib.cal'
        yield 'lino_xl.lib.outbox'
        yield 'lino_xl.lib.products'
        #~ 'lino.modlib.journals',
        # yield 'lino.modlib.vat'
        # yield 'lino.modlib.ledger'
        # yield 'lino.modlib.sales'
        # yield 'lino.modlib.finan'
        yield 'lino.modlib.uploads'
        yield 'lino_book.projects.igen'

    demo_fixtures = 'std few_languages few_countries few_cities demo_ee demo demo2'.split(
    )

SITE = Site(globals())



# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
This is the main module of the :ref:`belref` project.

.. autosummary::
   :toctree:

   models
   settings
   settings.demo
   settings.doctests


"""




from lino.projects.std.settings import *

class Site(Site):

    languages = 'en de fr'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        yield 'lino.modlib.system'
        yield 'lino.modlib.users'
        yield 'lino_book.projects.pisa'

SITE = Site(globals())




# -*- coding: UTF-8 -*-
# Copyright 2013-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""
Demo data specific for :ref:`voga`.
See also :doc:`/specs/courses`.
"""

from __future__ import unicode_literals

from builtins import range
from builtins import str

from lino.utils.instantiator import Instantiator, i2d
from lino.utils import Cycler
from django.conf import settings
from atelier.utils import date_offset
from lino.api import dd, rt, _
from lino.utils.mldbc import babeld

ENROLMENT_STORIES = Cycler([
    [(-20, None, None)],  # early enrolment
    [(-5, None, None)],  # normal enrolment
    [(-5, None, 5*7)],  # stops after 5 weeks
    [(None, None, None)],  # last minute enrolment (same day)
    [(15, 15, None)],  # starts after second week
    [(None, None, 3*7), (None, 7*7, None)]  # interrupts for 4 weeks
])


cal = dd.resolve_app('cal')
courses = dd.resolve_app('courses')
rooms = dd.resolve_app('rooms')

Booking = dd.resolve_model('rooms.Booking')
Room = dd.resolve_model('cal.Room')
Event = dd.resolve_model('cal.Event')
Partner = dd.resolve_model('contacts.Partner')
Company = dd.resolve_model('contacts.Company')
Teacher = dd.resolve_model('courses.Teacher')
TeacherType = dd.resolve_model('courses.TeacherType')
Pupil = dd.resolve_model('courses.Pupil')
Enrolment = dd.resolve_model('courses.Enrolment')
Course = dd.resolve_model('courses.Course')
Product = dd.resolve_model('products.Product')
CourseStates = courses.CourseStates
EnrolmentStates = courses.EnrolmentStates
BookingStates = rooms.BookingStates
Calendar = dd.resolve_model('cal.Calendar')
Tariff = rt.models.invoicing.Tariff

demo_date = dd.demo_date

# DEMO_REF_DATE = i2d(20140101)

# def demo_date(*args, **kw):
#     return date_offset(DEMO_REF_DATE, *args, **kw)


class Loader1(object):

    def objects(self):
        VatClasses = rt.models.vat.VatClasses
        # yield PupilType(ref="M", name="Mitglied")
        # yield PupilType(ref="H", name="Helfer")
        # yield PupilType(ref="L", name="LFV")
        # yield PupilType(ref="C", name="COK")
        #~ yield PupilType(ref="E",name="Extern")

        yield TeacherType(ref="S", **dd.str2kw('name', _("Independant")))
        yield TeacherType(ref="EP", **dd.babelkw('name', de="Ehrenamtlich pauschal", fr="Volontaire (forfait)", en="Voluntary (flat)"))
        yield TeacherType(ref="ER", **dd.babelkw('name', de="Ehrenamtlich real", fr="Volontaire (réel)", en="Voluntary (real)"))
        yield TeacherType(ref="LBA", **dd.babelkw('name', de="LBA", fr="ALE", en="LEA"))
        #~ yield TeacherType(ref="A",**dd.babelkw('name',de="Andere",fr="Autre",en="Other"))

        company = Instantiator('contacts.Company', 'name city:name').build

        we = company("Die Buche V.o.G.", "Eupen",
                     street="Birkenweg", street_no=5,
                     vat_id="BE0123456789")
        yield we
        settings.SITE.site_config.site_company = we
        yield settings.SITE.site_config

        ProductCat = rt.models.products.ProductCat
        # productcat = Instantiator('products.ProductCat').build

        self.course_fees = ProductCat(**dd.str2kw(
            'name', _("Participation fees")))
        yield self.course_fees

        self.trips = ProductCat(**dd.str2kw('name', _("Trips")))
        # et="Väljasõidud", de="Ausflüge", fr="Excursions"))
        yield self.trips

        kw = dd.str2kw('name', _("Journeys"))
        self.journeys_cat = ProductCat(**kw)
        yield self.journeys_cat

        kw.update(sales_price="295.00")
        self.journey_fee = Product(cat=self.journeys_cat, **kw)
        yield self.journey_fee

        rent = ProductCat(**dd.str2kw('name', _("Room renting")))
        # et="Ruumiüür", de="Raummiete", fr="Loyer"))
        yield rent
        # other = ProductCat(**dd.str2kw('name', _("Other")))
        # et="Muud", de="Sonstige", fr="Autres"))
        # yield other

        t5 = babeld(Tariff, _("5 times"), number_of_events=5, min_asset=1)
        yield t5
        t8 = babeld(Tariff, _("8 times"), number_of_events=8, min_asset=2)
        yield t8
        t12 = babeld(
            Tariff, _("12 times"), number_of_events=12, min_asset=4)
        yield t12

        product = Instantiator(
            'products.Product', "sales_price cat name",
            vat_class=VatClasses.services).build
        yield product("20", self.course_fees, "20€")
        yield product("48", self.course_fees, "48€/8 hours", tariff=t8)
        # yield p
        # yield Tariff(product=p, number_of_events=8, min_asset=2)

        yield product("64", self.course_fees, "64€/12 hours", tariff=t12)
        # yield p
        # yield Tariff(product=p, number_of_events=12, min_asset=4)

        yield product("50", self.course_fees, "50€/5 hours", tariff=t5)
        # yield p
        # yield Tariff(product=p, number_of_events=5, min_asset=1)

        yield product("80", self.course_fees, "80€")

        rent20 = product("20", rent, "Spiegelraum Eupen")
        yield rent20
        rent10 = product("10", rent, **dd.babelkw(
            'name',
            en="Rent per meeting", et="Ruumi üürimine",
            de="Raummiete pro Versammlung",
            fr="Loyer par réunion"))
        yield rent10

        self.PRICES = Cycler(Product.objects.filter(cat=self.course_fees))

        event_type = Instantiator('cal.EventType').build
        kw = dd.str2kw('name', _("Courses"))
        kw.update(dd.str2kw('event_label', _("Hour")))
        self.kurse = event_type(**kw)
        yield self.kurse
        settings.SITE.site_config.default_event_type = self.kurse
        yield settings.SITE.site_config

        self.seminare = event_type(**dd.str2kw('name', _("Seminars")))
        yield self.seminare

        self.excursions = event_type(
            max_days=10, **dd.str2kw('name', _("Excursions")))
        # de="Ausflüge",
        #  fr="Excursions",
        #  en="Excursions",
        yield self.excursions

        self.hikes = event_type(
            max_days=60, **dd.str2kw('name', _("Hikes")))
        # de="Wanderungen",
        # fr="Randonnées",
        # en="Hikes",
        yield self.hikes

        yield event_type(**dd.str2kw('name', _("Meetings")))
                                      # de="Versammlungen",
                                      # fr="Réunions",
                                      # en="Meetings",

        yield event_type(
            email_template='Team.eml.html',
            **dd.str2kw('name', _("Team Meetings")))
                         # de="Team-Besprechungen",
                         # fr="Coordinations en équipe",
                         # en="Team Meetings",

        #~ yield event_type(**dd.babelkw('name',
              #~ de="Feiertage",
              #~ fr="Jours fériés",
              #~ en="Holidays",
              #~ ))
        #~

        company = Instantiator('contacts.Company', 'name city:name').build
        eupen = company("Lern- und Begegnungszentrum", "Eupen",
                        street="Kirchstraße", street_no=39, street_box="/B2")
        yield eupen
        bbach = company("Lern- und Begegnungszentrum", "Butgenbach")
        yield bbach
        kelmis = company("Zur Klüüs", "Kelmis")
        yield kelmis
        stvith = company("Sport- und Freizeitzentrum", "Sankt Vith")
        yield stvith

        self.ext1 = company("AA Neudorf", "Raeren")
        yield self.ext1
        self.ext2 = company("Nisperter Schützenverein", "Eupen")
        yield self.ext2

        room = Instantiator('cal.Room').build
        kw = dict(company=eupen)
        kw.update(dd.str2kw('name', _("Mirrored room")))
        kw.update(fee=rent20)
        self.spiegel = room(**kw)
        yield self.spiegel

        kw.update(dd.str2kw('name', _("Computer room")))
        kw.update(fee=rent10)
        self.pc_eupen = room(**kw)
        yield self.pc_eupen

        kw = dict(company=bbach)
        kw.update(dd.str2kw('name', _("Conferences room")))
        self.konf = room(**kw)
        yield self.konf

        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_bbach = room(**kw)
        yield self.pc_bbach

        kw = dict(company=kelmis)
        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_kelmis = room(**kw)
        yield self.pc_kelmis

        kw = dict(company=stvith)
        kw.update(dd.str2kw('name', _("Computer room")))
        self.pc_stvith = room(**kw)
        yield self.pc_stvith

        # a room without company
        kw = dict()
        kw.update(dd.str2kw('name', _("Outside")))
        self.outside = room(**kw)
        yield self.outside

        COLORS = Cycler(Calendar.COLOR_CHOICES)

        for u in Room.objects.all():
            obj = Calendar(name=str(u), color=COLORS.pop())
            yield obj
            #~ dd.logger.info("20131018 %s", obj)
            u.calendar = obj
            u.save()


class Loader2(Loader1):

    def objects(self):

        yield super(Loader2, self).objects()

        Enrolment = rt.models.courses.Enrolment

        topic = Instantiator('courses.Topic').build
        line = Instantiator('courses.Line', 'topic event_type fee').build
        course = Instantiator(
            'courses.Course', 'line room start_time end_time').build
        booking = Instantiator(
            'rooms.Booking', 'room start_time end_time').build

        TEACHERS = Cycler(Teacher.objects.all())
        COMPANIES = Cycler(Company.objects.all())
        USERS = Cycler(settings.SITE.user_model.objects.all())

        def add_course(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(teacher=TEACHERS.pop())
            #~ kw.update(price=PRICES.pop())
            obj = course(*args, **kw)
            if obj.line.fee.tariff and obj.line.fee.tariff.number_of_events:
                obj.max_events = None
            return obj

        Product = rt.models.products.Product
        ProductCat = rt.models.products.ProductCat
        CourseAreas = rt.models.courses.CourseAreas
        PaymentTerm = rt.models.ledger.PaymentTerm

        journey_options = ProductCat(**dd.str2kw(
            'name', _("Hotel options")))
        yield journey_options
        option = Instantiator(Product, cat=journey_options).build
        yield option(**dd.str2kw('name', _("Single room")))
        yield option(**dd.str2kw('name', _("Double room")))
        yield option(**dd.str2kw('name', _("Triple room")))
        # yield option(**dd.str2kw('name', _("Shower")))
        # yield option(**dd.str2kw('name', _("Night club")))

        # trip_options = ProductCat(**dd.str2kw('name', _("Trip options")))
        # yield trip_options
        # option = Instantiator(Product, cat=trip_options).build
        # yield option(name="Eupen Oberstadt")
        # yield option(name="Eupen Unterstadt")
        # yield option(name="Raeren")
        # yield option(name="Kelmis")
        # yield option(name="Büllingen")

        journey = Instantiator(
            'courses.Course', 'line name start_date end_date').build

        def add_journey(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(teacher=TEACHERS.pop())
            kw.update(every_unit=cal.Recurrencies.once)
            kw.update(payment_term=PaymentTerm.get_by_ref('P30'))
            return journey(*args, **kw)

        self.journeys_topic = topic(**dd.str2kw('name', _("Journeys")))
        yield self.journeys_topic
        europe = line(self.journeys_topic,
                      self.excursions,
                      self.journey_fee,
                      options_cat=journey_options,
                      course_area=CourseAreas.journeys,
                      fees_cat=self.journeys_cat,
                      **dd.str2kw('name', _("Europe")))

        yield europe
        yield add_journey(europe, "Greece 2014",
                          i2d(20140814), i2d(20140820),
                          state=courses.CourseStates.active)
        yield add_journey(europe, "London 2014",
                          i2d(20140714), i2d(20140720))

        comp = topic(name="Computer")
        yield comp
        sport = topic(name="Sport")
        yield sport
        medit = topic(name="Meditation")
        yield medit
        externe = topic(name="Externe")
        yield externe

        obj = line(comp, self.kurse, self.PRICES.pop(),
                   fees_cat=self.course_fees,
                   ref="comp",
                   **dd.babelkw('name', de="Erste Schritte", en="First Steps"))
        yield obj
        kw = dict(max_events=8)
        kw.update(max_places=3)
        kw.update(start_date=demo_date(-430))
        kw.update(state=courses.CourseStates.active)
        kw.update(every=1)
        kw.update(every_unit=cal.Recurrencies.weekly)

        yield add_course(obj, self.pc_bbach, "13:30", "15:00",
                         monday=True, **kw)
        yield add_course(obj, self.pc_eupen, "17:30", "19:00",
                         wednesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "15:00",
                         friday=True, **kw)

        desc = """
    Behandelte Themengebiete:

    - Grundlagen, Voraussetzungen
    - Arbeiten im WWW unter Verwendung eines Browsers
    - Navigieren im WWW
    - Links in die Linkleiste legen
    - aus Webseiten heraus drucken
    - Favoriten bzw. Lesezeichen verwenden
    - Aufgabe und Funktionsweise von Suchmaschinen
    - Elektronische Post: E-Mails verfassen, senden, empfangen, beantworten
    - E-Mails mit Anlagen
    - E-mail Sicherheit
    - Tipps und Tricks
    """
        obj = line(
            comp, self.kurse, self.PRICES.pop(),
            ref="WWW",
            fees_cat=self.course_fees,
            description=desc, **dd.babelkw(
                'name',
                de="Internet: World Wide Web für Anfänger",
                en="Internet for beginners"))
        yield obj
        kw = dict(max_events=8)
        kw.update(max_places=4)
        kw.update(start_date=demo_date(-210))
        kw.update(state=courses.CourseStates.active)
        yield add_course(obj, self.pc_bbach, "13:30", "15:00",
                         monday=True, **kw)
        yield add_course(obj, self.pc_eupen, "17:30", "19:00",
                         wednesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "15:00",
                         friday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   ref="BT",
                   fees_cat=self.course_fees,
                   **dd.babelkw('name', de="Bauchtanz", en="Belly dancing"))
        yield obj
        kw = dict(max_events=8)
        kw.update(max_places=10)
        kw.update(start_date=demo_date(-420))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "19:00", "20:00",
                         wednesday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   ref="FG",
                   fees_cat=self.course_fees,
                   **dd.babelkw('name',
                                de="Funktionsgymnastik",
                                en="Functional gymnastics"))
        yield obj
        kw = dict(max_events=10, state=CourseStates.active)
        kw.update(max_places=5)
        kw.update(start_date=demo_date(-230))
        yield add_course(obj, self.spiegel, "11:00", "12:00", monday=True, **kw)
        yield add_course(obj, self.spiegel, "13:30", "14:30", monday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   ref="Rücken",
                   fees_cat=self.course_fees,
                   **dd.babelkw('name', de="Rücken fit durch Schwimmen", en="Swimming"))
        yield obj
        kw = dict(max_events=10, state=CourseStates.active)
        kw.update(max_places=20)
        kw.update(start_date=demo_date(50))
        yield add_course(obj, self.spiegel, "11:00", "12:00", monday=True, **kw)
        yield add_course(obj, self.spiegel, "13:30", "14:30", monday=True, **kw)
        yield add_course(obj, self.pc_stvith, "11:00", "12:00", tuesday=True, **kw)
        yield add_course(obj, self.pc_stvith, "13:30", "14:30", tuesday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "11:00", "12:00", thursday=True, **kw)
        yield add_course(obj, self.pc_kelmis, "13:30", "14:30", thursday=True, **kw)

        obj = line(sport, self.kurse, self.PRICES.pop(),
                   ref="SV",
                   fees_cat=self.course_fees,
                   **dd.babelkw('name', de="Selbstverteidigung im Alltag", en="Self-defence"))
        yield obj
        kw = dict(max_events=6)
        kw.update(max_places=12)
        kw.update(start_date=demo_date(-80))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "18:00", "19:00", friday=True, **kw)
        yield add_course(obj, self.spiegel, "19:00", "20:00", friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(),
                   ref="GLQ",
                   fees_cat=self.course_fees,
                   name="GuoLin-Qigong")
        yield obj
        kw = dict(max_events=10)
        kw.update(start_date=demo_date(-310))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.spiegel, "18:00", "19:30",
                         monday=True, **kw)
        yield add_course(obj, self.spiegel, "19:00", "20:30",
                         friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(),
                   ref="MED",
                   fees_cat=self.course_fees,
                   **dd.babelkw(
                       'name',
                       de="Den Kopf frei machen - zur inneren Ruhe finden",
                       en="Finding your inner peace"))
        yield obj
        kw = dict(max_events=10)
        kw.update(max_places=30)
        kw.update(start_date=demo_date(-610))
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.konf, "18:00", "19:30", monday=True, **kw)
        kw.update(start_date=demo_date(-110))
        yield add_course(obj, self.konf, "19:00", "20:30", friday=True, **kw)

        obj = line(medit, self.kurse, self.PRICES.pop(), name="Yoga")
        yield obj
        kw = dict(max_events=10)
        kw.update(start_date=demo_date(-560))
        kw.update(max_places=20)
        kw.update(state=CourseStates.active)
        yield add_course(obj, self.konf, "18:00", "19:30", monday=True, **kw)
        yield add_course(obj, self.konf, "19:00", "20:30", friday=True, **kw)

        for obj in Course.objects.filter(ref__isnull=True):
            if obj.line.fee.tariff and obj.line.fee.tariff.number_of_events:
                obj.ref = "%03dC" % obj.id
            else:
                obj.ref = "%03d" % obj.id
            yield obj

        EXTS = Cycler(self.ext1, self.ext2)

        def add_booking(*args, **kw):
            kw.update(user=USERS.pop())
            kw.update(event_type=self.seminare)
            #~ kw.update(price=PRICES.pop())
            #~ kw.update(fee=PRICES.pop())
            #~ kw.update(calendar=self.kurse)
            kw.update(every=1)
            kw.update(company=EXTS.pop())
            return booking(*args, **kw)

        #~ obj = line(externe,self.kurse,PRICES.pop(),**dd.babelkw('name',
            #~ de="Raumbuchung",en="Room booking"))
        #~ yield obj
        kw = dict(max_events=10)
        kw.update(every_unit=cal.Recurrencies.weekly)
        kw.update(start_date=demo_date(160))
        kw.update(state=BookingStates.registered)
        kw.update(company=COMPANIES.pop())
        yield add_booking(self.konf, "20:00", "22:00", tuesday=True, **kw)
        kw.update(company=COMPANIES.pop())
        yield add_booking(self.konf, "20:00", "22:00", thursday=True, **kw)

        kw = dict(max_events=1)
        kw.update(every_unit=cal.Recurrencies.once)
        kw.update(company=COMPANIES.pop())
        kw.update(every_unit=cal.Recurrencies.once)
        yield add_booking(self.konf, "10:00", "14:00", **kw)

        # a series of five week-ends:
        kw = dict()
        kw.update(user=USERS.pop())
        kw.update(teacher=TEACHERS.pop())
        kw.update(every_unit=cal.Recurrencies.monthly)
        kw.update(max_events=5)
        kw.update(friday=True)
        kw.update(payment_term=PaymentTerm.get_by_ref('P30'))
        yield journey(europe, "Five Weekends 2015",
                      i2d(20150619), i2d(20150621), **kw)

        # PUPILS = Cycler()
        #~ print 20130712, Pupil.objects.all()
        COURSES = Cycler(Course.objects.filter(
            line__fee__isnull=False).order_by('room__company__city'))
        # STATES = Cycler(EnrolmentStates.objects())
        FREE_EVENTS = Cycler([3, 2, 5, -2])

        # assert len(USERS) > 1
        pupils = Pupil.objects.order_by('id')
        # print("20171028 {} courses, {} pupils".format(
        #     len(COURSES), pupils.count()))
        # assert pupils.count() == 50

        n = 0
        for pupil in pupils:
            for i in range(pupil.id % 6):
                n += 1
                # every pupil can be in up to 5 courses. That's a bit
                # unrealistic, but we have 25 courses and 35
                # pupils and want to have
                course = COURSES.pop()

                def coursedate(*args, **kwargs):
                    return date_offset(course.start_date, *args, **kwargs)

                kw = dict(user=USERS.pop(), course=course, pupil=pupil)
                #~ print 20130712, kw
                story = ENROLMENT_STORIES.pop()
                assert isinstance(story, list)
                for rd, sd, ed in story:
                    # kw.update(state=STATES.pop())
                    # enrolments are requested from 3 weeks before until 1
                    # week after start date of course:
                    # kw.update(request_date=coursedate(i % 28 - 7))
                    kw.update(request_date=coursedate(rd))
                    if course.line == europe:
                        if n % 3 == 0:
                            kw.update(places=2)
                    obj = Enrolment(**kw)
                    # print("20171027b {}".format(obj))
                    obj.full_clean()
                    if sd is not None:
                        obj.start_date = coursedate(sd)
                        # obj.request_date = obj.start_date
                    if ed is not None:
                        obj.end_date = coursedate(ed)
                    if n % 10 == 0:
                        obj.state = EnrolmentStates.requested
                    elif n % 13 == 0:
                        obj.state = EnrolmentStates.cancelled
                    else:
                        obj.state = EnrolmentStates.confirmed
                    if n % 9 == 0:
                        if course.line.fee.tariff:
                            if course.line.fee.tariff.number_of_events:
                                obj.free_events = FREE_EVENTS.pop()
                    yield obj

        ses = settings.SITE.login()

        for model in (Course, Booking):
            for obj in model.objects.all():
                rc = ses.run(obj.do_update_events)
                if not rc.get('success', False):
                    raise Exception("update_reminders on %s returned %s" %
                                    (obj, rc))

        Event = rt.models.cal.Event
        EntryStates = rt.models.cal.EntryStates
        qs = Event.objects.filter(
            start_date__lt=dd.demo_date()).order_by('id')
        for i, e in enumerate(qs):
            if i % 8:
                e.state = EntryStates.took_place
                yield e

        # n = 0
        # for p in Partner.objects.all():
        #     if n > 10:
        #         break
        #     try:
        #         rc = ses.run(p.create_invoice)
        #         if rc.get('success', True):
        #             n += 1
        #     except Warning:
        #         pass


objects = Loader2().objects



# -*- coding: UTF-8 -*-
# Copyright 2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from lino.api import dd, _

class Bars(dd.ChoiceList):
    verbose_name = _("Bar")
    verbose_name_plural = _("Bars")

add = Bars.add_item
add('10', _("Sale"), 'sale')
add('20', _("Purchase"), 'purchase')
add('30', _("Profit"), 'profit')
add('40', _("Loss"), 'loss')

    



# -*- coding: UTF-8 -*-
# Copyright 2011-2013 Rumma & Ko Ltd

"""

"""

import logging
logger = logging.getLogger(__name__)

import os
import sys
import datetime

from dateutil import parser as dateparser

from django.conf import settings
from django.core.management import call_command
from django.db import IntegrityError
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand, CommandError

#~ from lino import lino_site
from lino.utils import dbfreader
from lino.utils import dblogger
#~ from lino import diag
#~ from lino.utils import crl2hex, hex2crl


from lino.utils import join_words
from lino_xl.lib.contacts.utils import street2kw
#~ from lino_xl.lib.contacts.models import GENDER_MALE, GENDER_FEMALE
from lino.api import dd, rt
from lino.utils.instantiator import Instantiator

from lino.core.utils import resolve_model, obj2str
import lino

from lino.utils import confirm, iif
from lino.core.utils import app_labels
from lino_book.projects.crl.models import CRL

Country = resolve_model('countries.Country')
Place = resolve_model('countries.Place')
Person = resolve_model("contacts.Person")
Company = resolve_model("contacts.Company")


def parsedate(T):
    if not T:
        return
    T = T.replace('.', '')
    try:
        if len(T) == 4:
            return (datetime.date(int(T), 6, 30), True)
        elif len(T) == 6:
            return (datetime.date(int(T[:4]), int(T[4:6]), 15), True)
        elif len(T) == 8:
            return (datetime.date(int(T[:4]), int(T[4:6]), int(T[6:])), False)
    except ValueError:
        pass
    dblogger.warning("Ignored invalid date value %r" % T)


def store(kw, **d):
    for k, v in d.items():
        if v is not None:
        # see `/blog/2011/0711`
        #~ if v:
            kw[k] = v

#~ def convert_username(name):
    #~ return name.lower()


def convert_sex(v):
    if v in ('W', 'F'):
        return 'F'
    if v == 'M':
        return 'M'
    return None


def isolang(x):
    if x == 'K':
        return 'et'
    if x == 'E':
        return 'en'
    if x == 'D':
        return 'de'
    if x == 'F':
        return 'fr'
    if x == 'N':
        return 'nl'


def is_company(data):
    # wer eine nationalregisternummer hat ist eine Person, selbst wenn er auch
    # eine MWst-Nummer hat.
    if data.get('NB2', False):
        return False
    if data.get('NOTVA', False):
    #~ if data.get('NOTVA',False):
        return True
    return False


def store_date(row, obj, rowattr, objattr):
    v = row[rowattr]
    if v:
        if isinstance(v, basestring):
            v = dateparser.parse(v)
        setattr(obj, objattr, v)


def ADR_id(cIdAdr):
    assert len(cIdAdr) == 3
    #~ assert [cIdAdr:-3] == '000'
    try:
        return 199000 + int(cIdAdr)
    except ValueError as e:
        return None


def country2kw(row, kw):
    # for both PAR and ADR

    if row.has_key('PROF'):
        activity = row['PROF']
        if activity:
            try:
                activity = int(activity)
            except ValueError:
                dblogger.debug("Ignored invalid value PROF = %r", activity)
            else:
                if activity:
                    try:
                        activity = Activity.objects.get(pk=activity)
                    except Activity.DoesNotExist:
                        activity = Activity(
                            id=activity, name=str(activity))
                        activity.save(force_insert=True)
                    kw.update(activity=activity)

    country = row['PAYS']
    if country:
        try:
            country = Country.objects.get(short_code__exact=country)
        except Country.DoesNotExist:
            country = Country(isocode=country, name=country,
                              short_code=country)
            country.save()
        kw.update(country=country)

    store(kw,
          phone=row['TEL'],
          fax=row['FAX'],
          email=row['EMAIL'],
          )

    kw.update(street2kw(join_words(row['RUE'], row['RUENUM'], row['RUEBTE'])))

    zip_code = row['CP']
    if zip_code:
        kw.update(zip_code=zip_code)
        try:
            city = Place.objects.get(
                country=country,
                zip_code__exact=zip_code,
            )
            kw.update(city=city)
        except Place.DoesNotExist as e:
            city = Place(zip_code=zip_code, name=zip_code, country=country)
            city.save()
            kw.update(city=city)
            #~ logger.warning("%s-%s : %s",row['PAYS'],row['CP'],e)
        except Place.MultipleObjectsReturned as e:
            dblogger.warning("%s-%s : %s", row['PAYS'], row['CP'], e)


def par2person(row, person):
    person.is_active = iif(row['IDPRT'] == 'I', False, True)
    if row['IDPRT'] == 'S':
        person.is_cpas = True
    elif row['IDPRT'] == 'A':
        person.is_senior = True


def pxs2person(row, person):

    kw = {}
    store(kw,
          card_number=row['CARDNUMBER'],
          card_type=row.get('CARDTYPE', ''),      # 20110110
          card_issuer=row.get('CARDISSUER', ''),      # 20110110
          noble_condition=row.get('NOBLEECOND', ''),      # 20110110
          birth_place=row.get('BIRTHPLACE', ''),
          remarks2=row.get('MEMO', ''),
          gender=convert_sex(row['SEXE'])
          )
    for k, v in kw.items():
        setattr(person, k, v)

    par2person(row, person)

    if row['IDMUT']:
        try:
            person.health_insurance = Company.objects.get(
                pk=ADR_id(row['IDMUT']))
        except ValueError as e:
            dblogger.warning(u"%s : invalid health_insurance %r",
                             obj2str(person), row['IDMUT'])
        except Company.DoesNotExist as e:
            dblogger.warning(u"%s : health_insurance %s not found",
                             obj2str(person), row['IDMUT'])

    if row['APOTHEKE']:
        try:
            person.pharmacy = Company.objects.get(pk=int(row['APOTHEKE']))
        except ValueError as e:
            dblogger.warning(u"%s : invalid pharmacy %r",
                             obj2str(person), row['APOTHEKE'])
        except Company.DoesNotExist as e:
            dblogger.warning(u"%s : pharmacy %s not found",
                             obj2str(person), row['APOTHEKE'])

    nat = row['NATIONALIT']
    if nat:
        try:
            country = Country.objects.get(short_code__exact=nat)
        except Country.DoesNotExist:
            country = Country(isocode=nat, name=nat, short_code=nat)
            country.save()
        person.nationality = country

    store_date(row, person, 'GEBDAT', 'birth_date')
    store_date(row, person, 'VALID1', 'card_valid_from')
    store_date(row, person, 'VALID2', 'card_valid_until')


def try_full_clean(i):
    while True:
        try:
            i.full_clean()
        except ValidationError as e:
            if not hasattr(e, 'message_dict'):
                raise
            for k in e.message_dict.keys():
                fld = i._meta.get_field(k)
                v = getattr(i, k)
                setattr(i, k, fld.default)
                dblogger.warning(
                    "%s : ignoring value %r for %s : %s", obj2str(i), v, k, e)
        return


def load_dbf(tableName, load):
    fn = os.path.join(settings.SITE.legacy_data_path, '%s.DBF' % tableName)
    f = dbfreader.DBFFile(fn, codepage="cp850")
    logger.info("Loading %d records from %s...", len(f), fn)
    f.open()
    for dbfrow in f:
        i = load(dbfrow)
        if i is not None:
            i = settings.TIM2LINO_LOCAL(tableName, i)
            if i is not None:
                try_full_clean(i)
                yield i

                #~ try:
                    #~ i.save()
                #~ except Exception,e:
                    #~ dblogger.warning("Failed to save %s from %s : %s",obj2str(i),dbfrow,e)
                    #~ dblogger.exception(e)
    f.close()


def load_O_(row):
    kw = {}
    o = row['O']
    if o:
        if len(o) == 2:
            try:
                country = Country.objects.get(pk=o)
                if country.name.upper() != row['A'].upper():
                    logger.warning('Country %s : %r != %r',
                                   o, country.name, row['A'])
            except Country.DoesNotExist:
                return Country(isocode=o, name=row['A'] + ' <<<<')
        elif len(o) == 4:
            try:
                be = Country.objects.get(pk='BE')
                city = Place.objects.get(country=be, zip_code=o)
                if city.name.upper() != row['A'].upper():
                    logger.warning('Place BE-%s : %r != %r',
                                   o, city.name, row['A'])
            except Place.MultipleObjectsReturned:
                logger.warning(
                    "O %s (%s) : MultipleObjectsReturned", o, row['A'])
            except Place.DoesNotExist:
                return Place(country=be, zip_code=o, name=row['A'] + ' <<<<')
        else:
            logger.warning("O %s (%s) : unknown format", o, row['A'])


def load_P_(row):
    kw = {}
    #~ kw.update(street2kw(join_words(row['RUE'],row['RUENUM'],row['RUEBTE'])))
    store(kw, last_name=row['AN'])
    store(kw, first_name=row['AP'])
    store(kw, crl=CRL(row['P'].encode('cp437')))
    #~ store(kw,crl=crl2hex(row['P']))
    OU = row['OU']
    if OU:
        kw.update(street2kw(OU))
    title = row['PQ']
    if title:
        if title == 'Mme':
            kw.update(language='fr', gender=dd.Genders.female)
        elif title == 'Mlle':
            kw.update(language='fr', gender=dd.Genders.female)
        elif title == 'M.':
            kw.update(language='fr', gender=dd.Genders.male)
        elif title == 'dHr':
            kw.update(language='nl', gender=dd.Genders.male)
        elif title == 'Mvw':
            kw.update(language='nl', gender=dd.Genders.female)
        elif title == 'Mr':
            kw.update(language='en', gender=dd.Genders.male)
        elif title == 'Mrs':
            kw.update(language='en', gender=dd.Genders.female)
        elif title == 'Hrrn':
            kw.update(language='de', gender=dd.Genders.male)
        elif title == 'Fr':
            kw.update(language='de', gender=dd.Genders.female)
        elif title == 'Fr.':
            kw.update(language='fr', gender=dd.Genders.male, title=u"Frère")
        elif title == 'Frl':
            kw.update(language='de', gender=dd.Genders.female)
        elif title == 'Bx':
            kw.update(gender=dd.Genders.male, title="Bx")
        elif title == 'Bse':
            kw.update(gender=dd.Genders.female, title="Bse")
        elif title == 'St':
            kw.update(gender=dd.Genders.male, title="St")
        elif title == 'Ste':
            kw.update(gender=dd.Genders.female, title="Ste")
        else:
            dblogger.warning("Ignored PQ value %r" % title)

    a = parsedate(row['T'])
    if a:
        kw.update(birth_date=a[0], birth_date_circa=a[1])
    a = parsedate(row['T'])
    if a:
        kw.update(died_date=a[0])
        if a[1]:
            logger.warning("Ignored 'circa' flag for died_date")
    return Person(**kw)


def objects():
    for i in load_dbf('P_', load_P_):
        yield i
    for i in load_dbf('O_', load_O_):
        yield i



# import os ; print "20161219 %s (pid:%s)" % (__name__, os.getpid())

import datetime

from ..settings import *


class Site(Site):
    the_demo_date = datetime.date(2017, 3, 16)
    languages = "en et"

    # default_ui = 'lino_extjs6.extjs6'
    
    # default_ui = 'lino.modlib.bootstrap3'
    # default_user = 'anonymous'
    
SITE = Site(globals())
# print "20161219 b"
DEBUG = True

# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from lino.api import dd
from lino_book.projects.de_BE.models import Expression


def O(fr, en, de, de_BE):
    return Expression(**dd.babelkw('designation', de=de, de_BE=de_BE, en=en, fr=fr))


def objects():
    yield O("l'atelier","the workshop","die Werkstatt","das Atelier")
    yield O("le camion","the lorry","der Lastwagen","der Camion")
    yield O("le classeur","the folder","der Ordner","die Farde")

    # TODO: non-ascii chars cause doctest failure
    yield O("le frigo","the fridge","der Kühlschrank","der Frigo")
    yield O("C'est ça l'important.","That's what it depends on.","Darauf kommt es an.","Darauf kommt es sich an.")
    yield O("C'est à propos de la santé","It's about health.","Es geht um die Gesundheit.","Es geht sich um die Gesundheit.")



from lino.api import dd

DATA = [
    ["Belgium", "Eupen", 17000],
    ["Belgium", "Liege", 400000],
    ["Belgium", "Raeren", 5000],
    ["Estonia", "Tallinn", 400000],
    ["Estonia", "Vigala", 1500],
]


class MyBase(dd.VirtualTable):

    @classmethod
    def get_data_rows(cls, ar):
        return DATA


    @dd.displayfield("Country")
    def country(cls, row, ar):
        return row[0]


class Cities(MyBase):

    column_names = "country city"

    @dd.displayfield("City")
    def city(cls, row, ar):
        return row[1]


class CitiesAndInhabitants(Cities):

    column_names = "country city population"

    @dd.displayfield("Population")
    def population(cls, row, ar):
        return row[2]





# -*- coding: UTF-8 -*-
# Copyright 2013-2016 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from __future__ import unicode_literals

from lino.projects.std.settings import *

from django.utils.translation import ugettext_lazy as _


class Site(Site):

    verbose_name = "Lino Belref"
    description = _("Belgian Reference")
    version = "0.1"
    url = "http://www.lino-framework.org/examples/belref.html"
    author = 'Luc Saffre'
    author_email = 'luc.saffre@gmail.com'

    demo_fixtures = 'demo all_countries be inscodes'

    #~ admin_prefix = 'admin'
    # default_ui = 'lino.modlib.bootstrap3'

    # anonymous_user_type = '900'

    languages = 'fr nl de'

    def get_installed_apps(self):
        yield super(Site, self).get_installed_apps()
        # yield 'lino.modlib.bootstrap3'
        #~ yield 'lino.modlib.gfks'
        #~ yield 'lino.modlib.users'
        yield 'lino.modlib.system'
        yield 'lino_xl.lib.statbel.countries'
        #~ yield 'lino_xl.lib.contacts'
        #~ yield 'lino_xl.lib.outbox'
        #~ yield 'lino_xl.lib.blogs'
        #~ yield 'lino.modlib.tickets'
        #~ yield 'lino_xl.lib.pages'
        yield 'lino_xl.lib.concepts'
        yield 'lino_book.projects.belref'

    def setup_menu(self, user_type, main):
        """
        We create a new menu from scratch because the default menu structure
        wouldn't fit.
        """
        from lino.api import dd
        mg = dd.plugins.concepts
        m = main.add_menu(mg.app_label, mg.verbose_name)
        m.add_action(self.modules.concepts.Concepts)
        m.add_action(self.modules.countries.Countries)
        m.add_action(self.modules.countries.Places)

    # def get_main_action(self, user):
    #     return self.modules.belref.Main.default_action



# Copyright 2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""
Lino Vilma extension of :mod:`lino_xl.lib.contacts`.

.. autosummary::
   :toctree:

    models

"""

from lino_xl.lib.contacts import Plugin


class Plugin(Plugin):
    
    extends_models = ['Person', 'Company']




# -*- coding: UTF-8 -*-
# Copyright 2016-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

from django.db import models
from lino.api import dd, _
from lino.mixins import BabelDesignated
from .choicelists import Bars

class Foo(BabelDesignated):

    class Meta(object):
        app_label = 'dumps'
    #     verbose_name = _("Foo")
    #     verbose_name_plural = _("Foos")

    last_visit = models.DateTimeField(_("Last visit"), editable=False)
    bar = Bars.field(default='sale')
    
class Foos(dd.Table):
    model = Foo





#!/usr/bin/env python
import os
import sys
if __name__ == "__main__":
    os.environ['DJANGO_SETTINGS_MODULE'] = 'lino_book.projects.chooser.settings'
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



from lino.projects.std.settings import *


class Site(Site):

    demo_fixtures = "std demo demo2"
    languages = 'en'
    # default_user = "robin"
    user_types_module = 'lino_xl.lib.xl.user_types'
    workflows_module = "lino_book.projects.workflows.entries.workflows"

    def get_installed_apps(self):

        yield super(Site, self).get_installed_apps()
        yield 'lino_xl.lib.contacts'
        yield 'lino.modlib.users'

        yield 'lino_book.projects.workflows.entries'


SITE = Site(globals())
# SITE.user_types_module = None

DEBUG = True



from django.db import models
from django.utils.translation import ugettext_lazy as _
   

class Person(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class AutoPerson(models.Model):
    id = models.AutoField(primary_key=True, verbose_name=_("ID"))
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class IntegerPerson(models.Model):
    id = models.IntegerField(primary_key=True, verbose_name=_("ID"))
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



class CharPerson(models.Model):
    id = models.CharField(_("ID"), primary_key=True, max_length=10)
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name



# -*- coding: UTF-8 -*-
# Copyright 2016-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

import datetime

from ..settings import *


class Site(Site):
    the_demo_date = datetime.date(2015, 5, 23)
    languages = "en de fr"
    use_ipdict = True

    # legacy_data_path = '...'

    # def get_installed_apps(self):
    #     yield super(Site, self).get_installed_apps()
    #     yield 'lino_xl.lib.tim2lino'

SITE = Site(globals())
DEBUG = True


# SITE.plugins.tim2lino.configure(
#     languages='de fr',
#     timloader_module='lino_xl.lib.tim2lino.spzloader',
#     dbf_table_ext='.FOX',
#     #use_dbf_py=True,
#     use_dbfread=True)


# the following line should not be active in a checked-in version
# DATABASES['default']['NAME'] = ':memory:'



from .demo import *
SITE = Site(
    globals(),
    remote_user_header='REMOTE_USER')
DEBUG = True
# SITE.appy_params.update(raiseOnError=True)
# SITE.appy_params.update(pythonWithUnoPath='/usr/bin/python3')
SITE.default_build_method = "appyodt"
SITE.webdav_url = '/'



from django.db import models
from lino.api import dd, _

YEAR_IN_SCHOOL_CHOICES = (
    ('FR', 'Freshman'),
    ('SO', 'Sophomore'),
    ('JR', 'Junior'),
    ('SR', 'Senior'),
    ('GR', 'Graduate'),
)

MENU = [
    # name reserved_for
    ('Potato', None),
    ('Vegetable', 'SO JR SR GR'),
    ('Meat', 'JR SR GR'),
    ('Fish', 'SR GR'),
]

year_in_school = models.CharField(
        max_length=2, choices=YEAR_IN_SCHOOL_CHOICES, blank=True)
food = models.CharField(max_length=20, blank=True)

@dd.chooser(simple_values=True)
def food_choices(cls, year_in_school):
    food = []
    for name, reserved_for in MENU:
        if (year_in_school is None) or (reserved_for is None) or year_in_school in reserved_for:
            food.append(name)
    return food


# -*- coding: UTF-8 -*-
# Copyright 2016-2017 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Adds customized urlpatters for a public restful interface.

This is currently just a proof of concept and will be active only when
:attr:`use_restful <lino.core.site.Site.use_restful>` is set to True.

"""

from __future__ import unicode_literals
from django.conf.urls import url, include
from lino.api import rt
# from lino_noi.libdjango.contrib.auth.models import User

from lino.core.urls import urlpatterns

from rest_framework import serializers, viewsets, routers

Ticket = rt.models.tickets.Ticket
User = rt.models.users.User

# Serializers define the API representation.
class TicketSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Ticket
        fields = ('id', 'user', 'end_user' ) # , 'state')

class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = ('username', 'first_name') # , 'state')


# ViewSets define the view behavior.
class TicketViewSet(viewsets.ModelViewSet):
    queryset = Ticket.objects.all()
    serializer_class = TicketSerializer

class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.all()
    serializer_class = UserSerializer


# Routers provide a way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'tickets', TicketViewSet)
router.register(r'users', UserViewSet)


# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browsable API.
urlpatterns += [
    url(r'^', include(router.urls)),
    url(r'^api-auth/', include('lino.modlib.restful.urls', namespace='rest_framework'))
]



# -*- coding: utf-8 -*-
# Copyright 2015 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)

"""Test certain aspects of `birth_date`.

This module is part of the Lino test suite. You can test only this
module by issuing either::

  $ go min2
  $ python manage.py test tests.test_birth_date

or::

  $ go lino
  $ python setup.py test -s tests.ProjectsTests.test_min2


"""

from __future__ import unicode_literals
from __future__ import print_function

from lino.api import rt

from lino.utils import IncompleteDate
from etgen.html import tostring
from lino.utils.mti import insert_child, delete_child
from lino.utils.djangotest import RemoteAuthTestCase
from django.core.exceptions import ValidationError
from django.utils import translation


def create(m, **kw):
    obj = m(**kw)
    obj.full_clean()
    obj.save()
    obj.after_ui_save(None, None)
    return obj


class QuickTest(RemoteAuthTestCase):

    fixtures = ['std']

    def test_this(self):

        Person = rt.models.contacts.Person
        Link = rt.models.humanlinks.Link
        LinkTypes = rt.models.humanlinks.LinkTypes
        LinksByHuman = rt.models.humanlinks.LinksByHuman

        father = create(
            Person, first_name="John", last_name="Doe",
            gender="M",
            birth_date='1980-07-31')
        try:
            son = create(
                Person, first_name="Joseph", last_name="Doe",
                gender="M",
                birth_date=IncompleteDate(2009, 2, 30))
        except ValidationError:
            pass
        else:
            self.fail("Expected ValidationError")
        son = create(
            Person, first_name="Joseph", last_name="Doe",
            gender="M",
            birth_date='2009-02-28')
        create(Link, parent=father, child=son, type=LinkTypes.parent)

        mary = create(
            Person, first_name="Mary", last_name="Doe",
            gender="F",
            birth_date='2010-01-30')
        create(Link, parent=father, child=mary, type=LinkTypes.parent)

        self.assertEqual(Person.objects.count(), 3)

        ar = LinksByHuman.request(father)
        s = ar.to_rst()
        # print(s)
        self.assertEquivalent("""

        John is Father of *Mary* (4 years) Father of *Joseph* (5
        years) """, s)

        # Create relationship as **Father**/**Son** **Adoptive
        # father**/**Adopted son** **Foster father**/**Foster son**
        # **Husband** **Partner** **Stepfather**/**Stepson** **Brother**
        # **Cousin** **Uncle**/**Nephew** **Relative** **Other** """, s)

        with translation.override('de'):
            ar = LinksByHuman.request(father)
            s = ar.to_rst()
            # print(s)
            self.assertEquivalent("""
            
            John ist Vater von *Mary* (4 Jahre) Vater von *Joseph* (5
            Jahre) """, s)

            # Beziehung erstellen als **Vater**/**Sohn**
            # **Adoptivvater**/**Adoptivsohn**
            # **Pflegevater**/**Pflegesohn** **Ehemann** **Partner**
            # **Stiefvater**/**Stiefsohn** **Bruder** **Vetter**
            # **Onkel**/**Neffe** **Verwandter** **Sonstiger** """, s)

        with translation.override('fr'):
            ar = LinksByHuman.request(father)
            s = ar.to_rst()
            # print(s)
            self.assertEquivalent(u"""

            John est Père de *Mary* (4 ans) Père de *Joseph* (5
            ans) """, s)

            # Créer lien de parenté en tant que **Père**/**Fils** **Père
            # adoptif**/**Fils adoptif** **Père nourricier**/**Fils
            # nourricier** **Mari** **Partenaire**
            # **Beau-père**/**Beau-fils** **Frère** **Cousin**
            # **Oncle**/**Nephew** **Parent** **Autre** """, s)

        # Here we are just testing whether no exception is risen. The
        # ouptut itself is more thoroughly tested elsewhere.
        html = LinksByHuman.get_table_summary(father, ar)
        s = tostring(html)
        self.assertEqual(s[:5], '<div>')
        
    def test_02(self):
        """Was written for :ticket:`488` (Kann Person nicht mehr von
        Organisation entfernen (delete mti child with siblings)).

        """
        Person = rt.models.contacts.Person
        Company = rt.models.contacts.Company
        Partner = rt.models.contacts.Partner

        # 1 : does delete_child work in normal situation?
        john = create(Person, first_name="John", last_name="Doe")
        as_partner = Partner.objects.get(pk=john.pk)
        delete_child(as_partner, Person)
        self.assertEqual(Person.objects.count(), 0)
        self.assertEqual(Partner.objects.count(), 1)
        as_partner.delete()

        # 2 : delete_child with an mti sibling
        john = create(Person, first_name="John", last_name="Doe")
        as_partner = Partner.objects.get(pk=john.pk)
        insert_child(as_partner, Company, full_clean=True)
        # this is the sitation:
        self.assertEqual(Person.objects.count(), 1)
        self.assertEqual(Partner.objects.count(), 1)
        self.assertEqual(Company.objects.count(), 1)

        # the following failed before #488 was fixed:
        delete_child(as_partner, Company)




from django.db import models

from lino_xl.lib.countries.mixins import AddressLocation


class Company(AddressLocation):
    class Meta:
        verbose_name = "Company"
        verbose_name_plural = "Companies"

    name = models.CharField("Name", max_length=50)

    def __str__(self):
        return self.name


from .ui import *



from lino.api import dd
dd.logger.info("pinglog via dd.logger.info()")
import logging
logging.getLogger('django').info("pinglog to logger django")
logging.getLogger('lino').info("pinglog to logger lino")
logging.getLogger('lino.foo').info("pinglog to logger lino.foo")
logging.getLogger('foo.lino').info("pinglog to logger foo.lino")



import datetime

from ..settings import *


class Site(Site):
    """Defines and instantiates a demo version of Lino Noi."""

    the_demo_date = datetime.date(2015, 5, 23)

    languages = "en de fr"

    def setup_plugins(self):
        """Change the default value of certain plugin settings.

        - :attr:`excerpts.responsible_user
          <lino_xl.lib.excerpts.Plugin.responsible_user>` is set to
          ``'jean'`` who is both senior developer and site admin in
          the demo database.

        """
        super(Site, self).setup_plugins()
        self.plugins.excerpts.configure(responsible_user='jean')


team_db = DATABASES

SITE = Site(globals())

DATABASES = team_db




# -*- coding: UTF-8 -*-
# Copyright 2014-2019 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)
"""Defines and instantiates a demo version of Lino Noi."""

import datetime

from ..settings import *


class Site(Site):

    the_demo_date = datetime.date(2015, 5, 23)

    languages = "en de fr"
    # readonly = True

    use_linod = True
    # use_ipdict = True
    # use_websockets = True
    # social_auth_backends = []
    # use_experimental_features = True
    # default_ui = 'lino_extjs6.extjs6'
    # default_ui = 'lino.modlib.bootstrap3'
    # default_ui = 'lino_openui5.openui5'

    # def get_installed_apps(self):
    #     yield super(Site, self).get_installed_apps()
    #     yield 'lino_react.react'

    def get_plugin_configs(self):
        for i in super(Site, self).get_plugin_configs():
            yield i
        yield ('excerpts', 'responsible_user', 'jean')
        # yield ('memo', 'front_end', 'react')

    def setup_plugins(self):
        """Change the default value of certain plugin settings.

        - :attr:`excerpts.responsible_user
          <lino_xl.lib.excerpts.Plugin.responsible_user>` is set to
          ``'jean'`` who is both senior developer and site admin in
          the demo database.

        """
        super(Site, self).setup_plugins()
        # self.plugins.social_auth.configure(
        #     backends=['social_core.backends.github.GithubOAuth2'])
        # self.plugins.excerpts.configure(responsible_user='jean')
        if False:
            self.plugins.mailbox.add_mailbox(
                'mbox', "Luc's aaa mailbox",
                '/home/luc/.thunderbird/luc/Mail/Local Folders/aaa')


SITE = Site(globals())

# SITE.plugins.extjs6.configure(theme_name='theme-classic')
# SITE.plugins.extjs6.configure(theme_name='theme-classic-sandbox')
# SITE.plugins.extjs6.configure(theme_name='theme-aria')
# SITE.plugins.extjs6.configure(theme_name='theme-grey')
# SITE.plugins.extjs6.configure(theme_name='theme-crisp')
# SITE.plugins.extjs6.configure(theme_name='theme-crisp-touch')
# SITE.plugins.extjs6.configure(theme_name='theme-neptune')
# SITE.plugins.extjs6.configure(theme_name='theme-neptune-touch')
# SITE.plugins.extjs6.configure(theme_name='theme-triton')
# SITE.plugins.extjs6.configure(theme_name='ext-theme-neptune-lino')

#in etc/aliases
# comments: /home/tonis/mbox
#SITE.plugins.inbox.configure(mbox_path='/home/tonis/mbox')
#SITE.plugins.inbox.configure(comment_reply_addr='comments@localhost')
DEBUG = True
ALLOWED_HOSTS=["*"]

# the following line should not be active in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'

# SITE.update_settings(ALLOWED_HOSTS=["192.168.0.26","127.0.0.1"])
